#!/usr/bin/env python

##############################################################################
##
## This file is part of Sardana
##
## http://www.tango-controls.org/static/sardana/latest/doc/html/index.html
##
## Copyright 2011 CELLS / ALBA Synchrotron, Bellaterra, Spain
## 
## Sardana is free software: you can redistribute it and/or modify
## it under the terms of the GNU Lesser General Public License as published by
## the Free Software Foundation, either version 3 of the License, or
## (at your option) any later version.
## 
## Sardana is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU Lesser General Public License for more details.
## 
## You should have received a copy of the GNU Lesser General Public License
## along with Sardana.  If not, see <http://www.gnu.org/licenses/>.
##
##############################################################################

"""This module is part of the Python Pool library. It defines the base classes
for CounterTimer"""

__all__ = [ "PoolCounterTimer" ]

__docformat__ = 'restructuredtext'

from sardana import ElementType

from .poolbasechannel import PoolBaseChannel


class PoolCounterTimer(PoolBaseChannel):

    def __init__(self, **kwargs):
        kwargs['elem_type'] = ElementType.CTExpChannel
        PoolBaseChannel.__init__(self, **kwargs)
    
    # --------------------------------------------------------------------------
    # value
    # --------------------------------------------------------------------------
    
    def set_write_value(self, w_value, timestamp=None, propagate=1):
        """Sets a new write value for the value.

        :param w_value:
            the new write value for value
        :type w_value:
            :class:`~numbers.Number`
        :param propagate:
            0 for not propagating, 1 to propagate, 2 propagate with priority
        :type propagate:
            int"""        
        self._value.set_write_value(w_value, timestamp=timestamp,
                                    propagate=propagate)




#!/usr/bin/env python

##############################################################################
##
## This file is part of Sardana
##
## http://www.tango-controls.org/static/sardana/latest/doc/html/index.html
##
## Copyright 2011 CELLS / ALBA Synchrotron, Bellaterra, Spain
## 
## Sardana is free software: you can redistribute it and/or modify
## it under the terms of the GNU Lesser General Public License as published by
## the Free Software Foundation, either version 3 of the License, or
## (at your option) any later version.
## 
## Sardana is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU Lesser General Public License for more details.
## 
## You should have received a copy of the GNU Lesser General Public License
## along with Sardana.  If not, see <http://www.gnu.org/licenses/>.
##
##############################################################################

""" """

__docformat__ = 'restructuredtext'

import PyTango.constants


def __init_pytango_devfailed(mod):
    consts = (
"API_AttrConfig", "API_AttrEventProp", "API_AttrIncorrectDataNumber", 
"API_AttrNoAlarm", "API_AttrNotAllowed", "API_AttrNotFound", 
"API_AttrNotWritable", "API_AttrOptProp", "API_AttrPropValueNotSet",
"API_AttrValueNotSet", "API_AttrWrongDefined", "API_AttrWrongMemValue",
"API_BadConfigurationProperty", "API_BlackBoxArgument", "API_BlackBoxEmpty",
"API_CannotCheckAccessControl", "API_CannotOpenFile",
"API_CantActivatePOAManager", "API_CantCreateClassPoa",
"API_CantCreateLockingThread", "API_CantFindLockingThread",
"API_CantGetClientIdent", "API_CantGetDevObjectId",
"API_CantInstallSignal", "API_CantRetrieveClass", "API_CantRetrieveClassList",
"API_CantStoreDeviceClass", "API_ClassNotFound",
"API_CmdArgumentTypeNotSupported", "API_CommandNotAllowed",
"API_CommandNotFound", "API_CorbaSysException", "API_CorruptedDatabase",
"API_DatabaseAccess", "API_DeviceLocked", "API_DeviceNotFound",
"API_DeviceNotLocked", "API_DeviceUnlockable", "API_DeviceUnlocked",
"API_EventSupplierNotConstructed", "API_IncoherentDbData",
"API_IncoherentDevData", "API_IncoherentValues", "API_IncompatibleAttrDataType",
"API_IncompatibleCmdArgumentType", "API_InitMethodNotFound",
"API_InitNotPublic", "API_InitThrowsException",
"API_JavaRuntimeSecurityException", "API_MemoryAllocation",
"API_MethodArgument", "API_MethodNotFound", "API_MissedEvents",
"API_NotSupportedFeature", "API_NtDebugWindowError",
"API_OverloadingNotSupported", "API_PolledDeviceNotInPoolConf",
"API_PolledDeviceNotInPoolMap", "API_PollingThreadNotFound",
"API_ReadOnlyMode", "API_SignalOutOfRange", "API_SystemCallFailed",
"API_WAttrOutsideLimit", "API_WizardConfError", "API_WrongEventData",
"API_WrongHistoryDataBuffer", "API_WrongLockingStatus", "API_ZmqInitFailed") 

    for const in consts:
        setattr(mod, const, const)

def __prepare_pytango():
    
    if not hasattr(PyTango.constants, "API_DeviceNotFound"):
        __init_pytango_devfailed(PyTango.constants)

__prepare_pytango()



#!/usr/bin/env python
# -*- coding: utf-8 -*-

##############################################################################
##
## This file is part of Sardana
## 
## http://www.tango-controls.org/static/sardana/latest/doc/html/index.html
##
## Copyright 2011 CELLS / ALBA Synchrotron, Bellaterra, Spain
## 
## Sardana is free software: you can redistribute it and/or modify
## it under the terms of the GNU Lesser General Public License as published by
## the Free Software Foundation, either version 3 of the License, or
## (at your option) any later version.
## 
## Sardana is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU Lesser General Public License for more details.
## 
## You should have received a copy of the GNU Lesser General Public License
## along with Sardana.  If not, see <http://www.gnu.org/licenses/>.
##
##############################################################################

"""Parameter parsing"""

__all__ = ['AbstractParam', 'Param', 'ParamRepeat', 'ParamList']
           
import sys

class AbstractParam:
    
    def __init__(self, name=None, desc=None):
        self.name     = name
        self.desc     = desc
        
    def getParam(self,idx=0):
        return self

    def getParamStr(self):
        return "<"+self.name+">"

    def getParamDescr(self):
        str = self.name + " (" + self.type + ") - " + self.desc
        if self.defvalue != 'None':
            str = str + "\nDefault value: " + self.defvalue 
        return str

    def getParamCount(self):
        return 1

    def formatParamValue(self,value):
        return value
    
    def __str__(self):
        return "%s(%s)" % (self.__class__.__name__, self.name)

class Param(AbstractParam):
    
    def __init__(self, name=None, type_name=None, desc=None, defvalue=None,
                 from_array=None):
        AbstractParam.__init__(self,name=name,desc=desc)
        self.type     = type_name
        self.defvalue = defvalue

        if from_array is not None:
            for key in ['name', 'type', 'desc', 'defvalue']:
                setattr(self, key, from_array.read())
    
    def formatParamValue(self,value):
        ret = value
        if self.type == "File":
            f = open(ret,'r')
            ret = f.read()
            f.close()
        return ret
    
          
class ParamRepeat(AbstractParam):
    def __init__(self, name=None, desc=None, opts=None, param_def=None,
                 from_array=None):
        AbstractParam.__init__(self,name=name,desc=desc)
        self.opts      = opts
        self.param_def = param_def

        if from_array is not None:
            self.name = from_array.read()
            type_name = from_array.read() 
            if type_name != 'ParamRepeat':
                msg = 'Expecting "ParamRepeat" type, got ' + type_name
                raise ValueError, msg
            self.desc = from_array.read()
            opt_str = from_array.read()
            opt_list = opt_str.split(', ')
            opts = {}
            for opt_str in opt_list:
                name, val = opt_str.split('=')
                if val == repr(None):
                    val = None
                else:
                    try:    val = int(val)
                    except: pass
                opts[name] = val
            self.opts = opts
            self.param_def = ParamList(from_array=from_array)

    def getParam(self,idx=0):
        idx = idx % len(self.param_def.pars)
        return self.param_def.getParam(idx)
        
    def getParamStr(self):
        return "[" + self.param_def.getParamStr() + " ]"

    def getParamDescr(self):
        return self.param_def.getParamDescr()
        
    def getParamCount(self):
        return self.param_def.getParamCount()
    
    def formatParamValue(self,value):
        return self.param_def.formatParamValue(value)

 
class ParamList(AbstractParam):
    
    def __init__(self, from_array):
        AbstractParam.__init__(self,name="ParamList",desc="List of parameters")
    
        self.pars = []
        self.has_param_repeat = False
        
        npars = int(from_array.read())
        for i in range(npars):
            if from_array[1] == 'ParamRepeat':
                par = ParamRepeat(from_array=from_array)
                self.has_param_repeat = True
            else:
                par = Param(from_array=from_array)
            self.pars.append(par)
    
    def getParam(self,idx=0):
        par_nb = len(self.pars)
        if idx < par_nb:
            par = idx
            return self.pars[idx].getParam(0)
        else:
            for i in range(par_nb):
                local_idx = idx - i
                local_nb = self.pars[i].getParamCount()
                if local_idx < local_nb:
                    return self.pars[i].getParam(local_idx)        

    def getParamStr(self):
        str = ""
        for par in self.pars:
            str = str + " " + par.getParamStr()
        return str

    def getParamDescr(self):
        str = ""
        for par in self.pars:
            str = str + "\n" + par.getParamDescr()
        return str

    def getParamCount(self):
        nb = 0
        for par in self.pars:
            local_nb = par.getParamCount()
            if local_nb == sys.maxint: 
                return sys.maxint
            nb += local_nb
        return nb
    
    def formatParamValue(self,value):
        
        # for now we are not able to handle ParamRepeat with special value format
        if self.has_param_repeat:
            return value
        
        for i,v in enumerate(value):
            value[i] = self.pars[i].formatParamValue(v)
        return value


#!/usr/bin/env python

##############################################################################
##
## This file is part of Sardana
##
## http://www.tango-controls.org/static/sardana/latest/doc/html/index.html
##
## Copyright 2011 CELLS / ALBA Synchrotron, Bellaterra, Spain
##
## Sardana is free software: you can redistribute it and/or modify
## it under the terms of the GNU Lesser General Public License as published by
## the Free Software Foundation, either version 3 of the License, or
## (at your option) any later version.
##
## Sardana is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU Lesser General Public License for more details.
##
## You should have received a copy of the GNU Lesser General Public License
## along with Sardana.  If not, see <http://www.gnu.org/licenses/>.
##
##############################################################################

"""This module contains the class definition for the MacroServer macro
manager"""

__all__ = ["MacroManager", "MacroExecutor", "is_macro"]

__docformat__ = 'restructuredtext'

import sys
import os
import inspect
import copy
import re
import functools
import traceback

from lxml import etree

from PyTango import DevFailed
from taurus.core.util.log import Logger
from taurus.core.util.codecs import CodecFactory

from sardana.sardanadefs import ElementType
from sardana.sardanamodulemanager import ModuleManager
from sardana.sardanaexception import format_exception_only_str
from sardana.sardanautils import is_pure_str, is_non_str_seq

from .msmanager import MacroServerManager
from .msmetamacro import MACRO_TEMPLATE, MacroLibrary, MacroClass, MacroFunction
from .msparameter import ParamDecoder
from .macro import Macro, MacroFunc
from .msexception import UnknownMacroLibrary, LibraryError, UnknownMacro, \
    MissingEnv, AbortException, StopException, MacroServerException

_BASE_DIR = os.path.dirname(os.path.abspath(__file__))


def islambda(f):
    """inspect doesn't come with islambda so I create one :-P"""
    return inspect.isfunction(f) and \
           f.__name__ == (lambda: True).__name__

def is_macro(macro, abs_file=None, logger=None):
    """Helper function to determine if a certain python object is a valid
    macro"""
    if inspect.isclass(macro):
        if not issubclass(macro, Macro):
            return False
        # if it is a class defined in some other module forget it to
        # avoid replicating the same macro in different macro files
        try:
            if inspect.getabsfile(macro) != abs_file:
                return False
        except TypeError:
            return False
    elif callable(macro) and not islambda(macro):
        # if it is a function defined in some other module forget it to
        # avoid replicating the same macro in different macro files
        try:
            if inspect.getabsfile(macro) != abs_file:
                return False
        except TypeError:
            return False

        if not hasattr(macro, 'macro_data'):
            return False

        args, varargs, keywords, _ = inspect.getargspec(macro)
        if len(args) == 0:
            if logger:
                logger.debug("Could not add macro %s: Needs at least one "
                             "parameter (usually called 'self')",
                             macro.func_name)
            return False
        if keywords is not None:
            if logger:
                logger.debug("Could not add macro %s: Unsupported keyword "
                             "parameters '%s'", macro.func_name, keywords)
            return False
        if varargs and len(args) > 1:
            if logger:
                logger.debug("Could not add macro %s: Unsupported giving "
                             "named parameters '%s' and varargs '%s'",
                             macro.func_name, args, varargs)
            return False
    else:
        return False
    return True


class MacroManager(MacroServerManager):

    DEFAULT_MACRO_DIRECTORIES = os.path.join(_BASE_DIR, 'macros'),

    def __init__(self, macro_server, macro_path=None):
        MacroServerManager.__init__(self, macro_server)
        if macro_path is not None:
            self.setMacroPath(macro_path)

    def reInit(self):
        if self.is_initialized():
            return

        # dict<str, MacroLibrary>
        # key   - module name (without path and without extension)
        # value - MacroLibrary object representing the module
        self._modules = {}

        # dict<str, <MacroClass>
        # key   - macro name
        # value - MacroClass object representing the macro
        self._macro_dict = {}

        # list<str>
        # elements are absolute paths
        self._macro_path = []

        # dict<Door, <MacroExecutor>
        # key   - door
        # value - MacroExecutor object for the door
        self._macro_executors = {}

        MacroServerManager.reInit(self)

    def cleanUp(self):
        if self.is_cleaned():
            return

        #if self._modules:
        #    ModuleManager().unloadModules(self._modules.keys())

        self._macro_path = None
        self._macro_dict = None
        self._modules = None

        MacroServerManager.cleanUp(self)

    def setMacroPath(self, macro_path):
        """Registers a new list of macro directories in this manager.
        Warning: as a consequence all the macro modules will be reloaded.
        This means that if any reference to an old macro object was kept it will
        refer to an old module (which could possibly generate problems of type
        class A != class A)."""
        p = []
        for item in macro_path:
            p.extend(item.split(":"))

        # filter empty and commented paths
        p = [ i for i in p if i and not i.startswith("#") ]

        # add basic macro directories
        for macro_dir in self.DEFAULT_MACRO_DIRECTORIES:
            if not macro_dir in p:
                p.append(macro_dir)

        self._macro_path = p

        macro_file_names = self._findMacroLibNames()
        for mod_name in macro_file_names:
            try:
                self.reloadMacroLib(mod_name)
            except:
                pass

    def getMacroPath(self):
        return self._macro_path

    def _findMacroLibName(self, lib_name, path=None):
        path = path or self.getMacroPath()
        f_name = lib_name
        if not f_name.endswith('.py'):
            f_name += '.py'
        for p in path:
            try:
                elems = os.listdir(p)
                if f_name in elems:
                    return os.path.abspath(os.path.join(p, f_name))
            except:
                self.debug("'%s' is not a valid path" % p)
        return None

    def _findMacroLibNames(self, path=None):
        path = path or self.getMacroPath()
        ret = {}
        for p in reversed(path):
            try:
                for f in os.listdir(p):
                    name,ext = os.path.splitext(f)
                    if name.startswith("_"):
                        continue
                    if ext.endswith('py'):
                        ret[name] = os.path.abspath(os.path.join(p, f))
            except:
                self.debug("'%s' is not a valid path" % p)
        return ret

    def _fromNameToFileName(self, lib_name, path=None):
        path = path or self.getMacroPath()[0]
        f_name = lib_name
        if not f_name.endswith('.py'):
            f_name += '.py'

        if os.path.isabs(f_name):
            path, _ = os.path.split(f_name)
            if not path in self.getMacroPath():
                raise Exception("'%s' is not part of the MacroPath" % path)
        else:
            f_name = os.path.join(path, f_name)
        return f_name

    def getOrCreateMacroLib(self, lib_name, macro_name=None):
        """Gets the exiting macro lib or creates a new macro lib file. If
        name is not None, a macro template code for the given macro name is
        appended to the end of the file.

        :param lib_name:
            module name, python file name, or full file name (with path)
        :type lib_name: str
        :param macro_name:
            an optional macro name. If given a macro template code is appended
            to the end of the file (default is None meaning no macro code is
            added)
        :type macro_name: str

        :return:
            a sequence with three items: full_filename, code, line number is 0
            if no macro is created or n representing the first line of code for
            the given macro name.
        :rtype: seq<str, str, int>"""
        # if only given the module name
        try:
            macro_lib = self.getMacroLib(lib_name)
        except UnknownMacroLibrary:
            macro_lib = None

        if macro_name is None:
            line_nb = 0
            if macro_lib is None:
                f_name, code = self.createMacroLib(lib_name), ''
            else:
                f_name = macro_lib.file_path
                f = file(f_name)
                code = f.read()
                f.close()
        else:
        # if given macro name
            if macro_lib is None:
                f_name, code, line_nb = self.createMacro(lib_name, macro_name)
            else:
                macro = macro_lib.get_macro(macro_name)
                if macro is None:
                    f_name, code, line_nb = self.createMacro(lib_name, macro_name)
                else:
                    _, line_nb = macro.code
                    f_name = macro.file_path
                    f = file(f_name)
                    code = f.read()
                    f.close()

        return [ f_name, code, line_nb ]

    def setMacroLib(self, lib_name, code, auto_reload=True):
        f_name = self._fromNameToFileName(lib_name)
        f = open(f_name, 'w')
        f.write(code)
        f.flush()
        f.close()
        _, name = os.path.split(f_name)
        mod, _ = os.path.splitext(name)
        if auto_reload:
            self.reloadMacroLib(mod)
        return mod

    def createMacroLib(self, lib_name, path=None):
        """Creates a new empty macro library (python module)"""
        f_name = self._fromNameToFileName(lib_name, path)

        if os.path.exists(f_name):
            raise Exception("Unable to create macro lib: '%s' already exists" % f_name)

        f = open(f_name, 'w')
        f.close()
        return f_name

    def createMacro(self, lib_name, macro_name):
        f_name = self._fromNameToFileName(lib_name)

        create = not os.path.exists(f_name)

        template = ''
        if create:
            template += 'from sardana.macroserver.macro import Macro, macro, Type\n\n'
            line_nb = 4
        else:
            template += '\n'
            t = open(f_name, 'rU')
            line_nb = -1
            for line_nb, _ in enumerate(t): pass
            line_nb += 3
            t.close()

        f = open(f_name, 'a+')
        f_templ = None
        try:
            dir_name = os.path.realpath(__file__)
            dir_name = os.path.dirname(dir_name)
            template_fname = 'macro_template.txt'
            template_fname = os.path.join(dir_name, template_fname)
            f_templ = open(template_fname, 'r')
            template += f_templ.read()
            f_templ.close()
        except:
            self.debug("Failed to open template macro file. Using simplified template")
            template += MACRO_TEMPLATE
            if f_templ:
                f_templ.close()

        template = template.replace('@macro_name@', macro_name)
        try:
            f.write(template)
            f.flush()
            f.seek(0)
            code = f.read()
        finally:
            f.close()
        return f_name, code, line_nb

    def reloadMacro(self, macro_name, path=None):
        """Reloads the module corresponding to the given macro name

        :raises: MacroServerExceptionList in case the macro is unknown or the
        reload process is not successful

        :param macro_name: macro name
        :param path: a list of absolute path to search for libraries
                     (optional, default=None, means the current MacroPath
                     will be used)"""
        self.reloadMacros([macro_name], path=path)

    def reloadMacros(self, macro_names, path=None):
        """Reloads the modules corresponding to the given macro names

        :raises: MacroServerExceptionList in case the macro(s) are unknown or the
        reload process is not successful

        :param macro_names: a list of macro names
        :param path: a list of absolute path to search for libraries (optional,
                     default=None, means the current MacroPath will be used)"""
        module_names = []
        for macro_name in macro_names:
            module_name = self.getMacro(macro_name).module_name
            module_names.append(module_name)
        self.reloadMacroLibs(module_names, path=path)

    def reloadMacroLibs(self, module_names, path=None):
        """Reloads the given lib(=module) names

        :raises: MacroServerExceptionList in case the reload process is not
        successful for at least one library

        :param module_names: a list of module names
        :param path: a list of absolute path to search for libraries
                     (optional, default=None, means the current MacroPath
                     will be used)"""
        ret = []
        for module_name in module_names:
            m = self.reloadMacroLib(module_name, path=path)
            if m: ret.append(m)
        return ret

    def reloadLib(self, module_name, path=None):
        """Reloads the given library(=module) names.

        :raises:
            ImportError in case the reload process is not successful
            LibraryError if trying to reload a macro library
            
        :param module_name: module name
        :param path:
            a list of absolute path to search for libraries [default: None,
            meaning search in MacroPath. If not found, search for a built-in,
            frozen or special module and continue search in sys.path. ]
        :return: the reloaded python module object"""
        
        if module_name in self._modules:
            raise LibraryError("Cannot use simple reload to reload a Macro Library")
        
        mod_manager = ModuleManager()
        retry = path is None
        try:
            if retry:
                path = self.getMacroPath()
                if path:
                    path = copy.copy(path)
                    path.reverse()                
            return mod_manager.reloadModule(module_name, path)
        except ImportError:
            if retry:
                return mod_manager.reloadModule(module_name, path=None)
            else:
                raise

    def reloadMacroLib(self, module_name, path=None):
        """Reloads the given library(=module) names.

        :raises:
            LibraryError in case the reload process is not successful

        :param module_name: macro library name (=python module name)
        :param path:
            a list of absolute path to search for libraries [default: None,
            means the current MacroPath will be used]
        :return: the MacroLibrary object for the reloaded macro library"""
        path = path or self.getMacroPath()
        # reverse the path order:
        # more priority elements last. This way if there are repeated elements
        # they first ones (lower priority) will be overwritten by the last ones
        if path:
            path = copy.copy(path)
            path.reverse()

        # if there was previous Macro Library info remove it
        old_macro_lib = self._modules.pop(module_name, None)
        if old_macro_lib is not None:
            for macro in old_macro_lib.get_macros():
                self._macro_dict.pop(macro.name)

        mod_manager = ModuleManager()
        m, exc_info = None, None
        try:
            m = mod_manager.reloadModule(module_name, path)
        except:
            exc_info=sys.exc_info()
        macro_lib = None

        params = dict(module=m, name=module_name,
                      macro_server=self.macro_server, exc_info=exc_info)
        if m is None:
            file_name = self._findMacroLibName(module_name)
            if file_name is None:
                if exc_info:
                    msg = format_exception_only_str(*exc_info[:2])
                else:
                    msg = "Error (re)loading macro library '%s'" % module_name
                raise LibraryError(msg, exc_info=exc_info)
            params['file_path'] = file_name
            macro_lib = MacroLibrary(**params)
        else:
            macro_lib = MacroLibrary(**params)
            abs_file = macro_lib.file_path
            _is_macro = functools.partial(is_macro, abs_file=abs_file,
                                          logger=self)
            for _, macro in inspect.getmembers(m, _is_macro):
                try:
                    self.addMacro(macro_lib, macro)
                except:
                    self.error("Error adding macro %s", macro.__name__)
                    self.debug("Details:", exc_info=1)
        self._modules[module_name] = macro_lib
        return macro_lib

    def addMacro(self, macro_lib, macro):
        add = self.addMacroFunction
        if inspect.isclass(macro):
            add = self.addMacroClass
        return add(macro_lib, macro)

    def addMacroClass(self, macro_lib, klass):
        macro_name = klass.__name__
        action = (macro_lib.has_macro(macro_name) and "Updating") or "Adding"
        self.debug("%s macro class %s" % (action, macro_name))

        params = dict(macro_server=self.macro_server, lib=macro_lib,
                      klass=klass)
        macro_class = MacroClass(**params)
        macro_lib.add_macro_class(macro_class)
        self._macro_dict[macro_name] = macro_class

    def addMacroFunction(self, macro_lib, func):
        macro_name = func.func_name
        action = (macro_lib.has_macro(macro_name) and "Updating") or "Adding"
        self.debug("%s macro function %s" % (action, macro_name))

        params = dict(macro_server=self.macro_server, lib=macro_lib,
                      function=func)
        macro_function = MacroFunction(**params)
        macro_lib.add_macro_function(macro_function)
        self._macro_dict[macro_name] = macro_function

    def getMacroLibNames(self):
        return sorted(self._modules.keys())

    def getMacroLibs(self, filter=None):
        if filter is None:
            return self._modules
        expr = re.compile(filter, re.IGNORECASE)
        ret = {}
        for name, macro_lib in self._modules.iteritems():
            if expr.match(name) is None:
                continue
            ret[name] = macro_lib
        return ret

    def getMacros(self, filter=None):
        """Returns a :obj:`dict` containing information about macros.

        :param filter:
            a regular expression for macro names [default: None, meaning all
            macros]
        :type filter: str
        :return: a :obj:`dict` containing information about macros
        :rtype:
            :obj:`dict`\<:obj:`str`\, :class:`~sardana.macroserver.msmetamacro.MacroCode`\>"""
        if filter is None:
            return self._macro_dict
        expr = re.compile(filter, re.IGNORECASE)

        ret = {}
        for name, macro in self._macro_dict.iteritems():
            if expr.match(name) is None:
                continue
            ret[name] = macro
        return ret
    
    def getMacroClasses(self, filter=None):
        """Returns a :obj:`dict` containing information about macro classes.

        :param filter:
            a regular expression for macro names [default: None, meaning all
            macros]
        :type filter: str
        :return: a :obj:`dict` containing information about macro classes
        :rtype:
            :obj:`dict`\<:obj:`str`\, :class:`~sardana.macroserver.msmetamacro.MacroClass`\>"""
        macros = self.getMacros(filter=filter)
        macro_classes = {}
        for name, macro in macros.items():
            if macro.get_type() == ElementType.MacroClass:
                macro_classes[name] = macro
        return macro_classes

    def getMacroFunctions(self, filter=None):
        """Returns a :obj:`dict` containing information about macro functions.

        :param filter:
            a regular expression for macro names [default: None, meaning all
            macros]
        :type filter: str
        :return: a :obj:`dict` containing information about macro functions
        :rtype:
            :obj:`dict`\<:obj:`str`\, :class:`~sardana.macroserver.msmetamacro.MacroFunction`\>"""
        macros = self.getMacros(filter=filter)
        macro_classes = {}
        for name, macro in macros.items():
            if macro.get_type() == ElementType.MacroFunction:
                macro_classes[name] = macro
        return macro_classes
        
    def getMacroNames(self):
        return sorted(self._macro_dict.keys())

    def getMacro(self, macro_name):
        ret = self._macro_dict.get(macro_name)
        if ret is None:
            raise UnknownMacro("Unknown macro %s" % macro_name)
        return ret

    def getMacroClass(self, macro_name):
        return self.getMacro(macro_name)

    def getMacroFunction(self, macro_name):
        return self.getMacro(macro_name)

    def removeMacro(self, macro_name):
        self._macro_dict.pop(macro_name)

    def getMacroLib(self, name):
        if os.path.isabs(name):
            abs_file_name = name
            for lib in self._modules.values():
                if lib.file_path == abs_file_name:
                    return lib
        elif name.count(os.path.extsep):
            file_name = name
            for lib in self._modules.values():
                if lib.file_name == file_name:
                    return lib
        module_name = name
        ret = self._modules.get(module_name)
        if ret is None:
            raise UnknownMacroLibrary("Unknown macro library %s" % module_name)
        return ret

    def getMacroCode(self, macro_name):
        return self.getMacro(macro_name).code_object

    def getMacroClassCode(self, macro_name):
        return self.getMacroClass(macro_name).klass

    def getMacroFunctionCode(self, macro_name):
        return self.getMacroFunction(macro_name).function

    def getMacroInfo(self, macro_names, format='json'):
        if isinstance(macro_names, (str, unicode)):
            macro_names = [macro_names]
        ret = []
        json_codec = CodecFactory().getCodec(format)
        for macro_name in macro_names:
            macro_meta = self.getMacro(macro_name)
            ret.append(json_codec.encode(('', macro_meta.serialize()))[1])
        return ret

    def decodeMacroParameters(self, door, in_par_list):
        if len(in_par_list) == 0:
            raise RuntimeError('Macro name not specified')
        macro_name = in_par_list[0]
        macro_meta = self.getMacro(macro_name)
        out_par_list = ParamDecoder(door, macro_meta, in_par_list)
        return macro_meta, in_par_list, out_par_list

    def strMacroParamValues(self,par_list):
        """strMacroParamValues(list<string> par_list) -> list<string>

           Creates a short string representantion of the parameter values list.
           Params:
               - par_list: list of strings representing the parameter values.
           Return:
               a list containning an abreviated version of the par_list argument.
        """
        ret = []
        for p in par_list:
            param_str = str(p)
            if len(param_str)>9:
                param_str = param_str[:9] + "..."
            ret.append(param_str)
        return ret

    def prepareMacro(self, macro_class, par_list, init_opts={}, prepare_opts={}):
        """Creates the macro object and calls its prepare method.
           The return value is a tuple (MacroObject, return value of prepare)
        """
        macro = self.createMacroObj(macro_class, par_list, init_opts=init_opts)
        prepare_result = self.prepareMacroObj(macro, par_list, prepare_opts=prepare_opts)
        return macro, prepare_result

    def createMacroObj(self, macro_class, par_list, init_opts={}):
        macro_env = macro_class.env
        macro_name = macro_class.__name__

        environment = init_opts.get('environment')

        r = []
        for env in macro_env:
            if not environment.has_env(env):
                r.append(env)
        if r:
            raise MissingEnv("The macro %s requires the following missing " \
                             "environment to be defined: %s"
                             % (macro_name, str(r)))

        macro_opts = {
            'no_exec': True,
            'create_thread' : True,
            'external_prepare' : True
        }

        macro_opts.update(init_opts)
        macroObj = macro_class(*par_list, **macro_opts)
        return macroObj

    def prepareMacroObj(self, macro, par_list, prepare_opts={}):
        return macro.prepare(*par_list, **prepare_opts)

    def createMacroObjFromMeta(self, meta, par_list, init_opts={}):
        code = meta.code_object
        macro_env = code.env or ()

        environment = init_opts.get('environment')

        r = []
        for env in macro_env:
            if not environment.has_env(env):
                r.append(env)
        if r:
            macro_name = meta.name
            raise MissingEnv("The macro %s requires the following missing " \
                             "environment to be defined: %s"
                             % (macro_name, str(r)))

        macro_opts = dict(no_exec=True, create_thread=True,
                          external_prepare=True)

        macro_opts.update(init_opts)
        if meta.get_type() == ElementType.MacroClass:
            macroObj = meta.macro_class(*par_list, **macro_opts)
        else:
            macro_opts['function'] = code
            macroObj = MacroFunc(*par_list, **macro_opts)
        return macroObj

    def getMacroExecutor(self, door):
        me = self._macro_executors.get(door)
        if me is None:
            self._macro_executors[door] = me = MacroExecutor(door)
        return me


class MacroExecutor(Logger):
    """ """

    class RunSubXMLHook:
        def __init__(self, me, xml):
            self._me = me
            self._xml = xml

        def __call__(self):
            self._me._runXMLMacro(xml=self._xml)

    def __init__(self, door):
        self._door = door
        self._macro_counter = 0

        # dict<PoolElement, set<Macro>>
        # key PoolElement - reserved object
        # value set<Macro> macros that reserved the object
        self._reserved_objs = {}

        # dict<Macro, seq<PoolElement>>
        # key Macro - macro object
        # value - sequence of reserverd objects by the macro
        self._reserved_macro_objs = {}

        # reset the stacks
#        self._macro_stack = None
#        self._xml_stack = None
        self._macro_stack = []
        self._xml_stack = []
        self._macro_pointer = None
        self._aborted = False
        self._stopped = False
        self._paused = False
        self._last_macro_status = None

        name = "%s.%s" % (str(door), self.__class__.__name__)
        self._macro_status_codec = CodecFactory().getCodec('json')
        self.call__init__(Logger, name)

    def getDoor(self):
        return self._door

    door = property(getDoor)

    def getMacroServer(self):
        return self.door.macro_server

    macro_server = property(getMacroServer)

    @property
    def macro_manager(self):
        return self.macro_server.macro_manager

    def getNewMacroID(self):
        self._macro_counter -= 1
        return self._macro_counter

    def _preprocessParameters(self, par_str_list):
        if not par_str_list[0].lstrip().startswith('<'):
            xml_root = xml_seq = etree.Element('sequence')
            xml_macro = etree.SubElement(xml_seq, 'macro', name=par_str_list[0])
            for p in par_str_list[1:]:
                etree.SubElement(xml_macro, 'param', value=p)
        else:
            xml_root = etree.fromstring(par_str_list[0])

        macro_nodes = xml_root.findall('.//macro')

        # make sure macros exist
        self.__checkXMLSequence(macro_nodes)

        # fill the xml with macro id macro_line
        self.__fillXMLSequence(macro_nodes)

        return xml_root

    def __checkXMLSequence(self, macros):
        for macro in macros:
            name = macro.get('name')
            self.macro_manager.getMacro(name)

    def __fillXMLSequence(self, macros):
        for macro in macros:
            eid = macro.get('id')
            if eid is None:
                eid = str(self.getNewMacroID())
                macro.set('id', eid)
            name = macro.get('name')
            params = []
            
            # SEEMS THERE IS A MEMORY LEAK IN lxml.etree Element.xpath :
            # https://bugs.launchpad.net/lxml/+bug/397933
            # https://mailman-mail5.webfaction.com/pipermail/lxml/2011-October/006205.html
            # We work around it using findall:
            
            #for p in macro.xpath('param|paramrepeat'):
            #    if p.tag == 'param':
            #        params.append(p.get('value'))
            #    else:
            #        params.extend([ p2.get('value') for p2 in p.findall(".//param")])
            for p in macro.findall('*'):
                if p.tag == 'param':
                    params.append(p.get('value'))
                elif p.tag == 'paramrepeat':
                    params.extend([ p2.get('value') for p2 in p.findall(".//param")])
            macro.set('macro_line', "%s(%s)" % (name, ", ".join(params)))

    def __preprocessResult(self, result):
        """decodes the given output from a macro in order to be able to send to
        the result channel as a sequence<str>

        :param out: output value

        :return: the output as a sequence of strings
        """
        if result is None:
            return ()
        if is_non_str_seq(result):
            result = map(str, result)
        else:
            result = (str(result),)
        return result

    def _decodeXMLMacroParameters(self, xml_macro):
        str_params = [xml_macro.get('name')]
        for param in xml_macro.findall('.//param'):
            str_params.append(param.get('value'))
        return self._decodeMacroParameters(str_params)

    def _decodeMacroParameters(self, params):
        return self.macro_manager.decodeMacroParameters(self.door, params)

    def _prepareXMLMacro(self, xml_macro, parent_macro=None):
        macro_meta, _, params = self._decodeXMLMacroParameters(xml_macro)
        init_opts = {
            'id'           : xml_macro.get('id'),
            'macro_line'   : xml_macro.get('macro_line'),
            'parent_macro' : parent_macro,
        }

        macro_obj = self._createMacroObj(macro_meta, params, init_opts)
        for macro in xml_macro.findall('macro'):
            hook = MacroExecutor.RunSubXMLHook(self, macro)
            hook_hints = macro.findall('hookPlace')
            if hook_hints is None:
                macro_obj.hooks = [ hook ]
            else:
                hook_places = [ h.text for h in hook_hints ]
                macro_obj.hooks = [ (hook, hook_places) ]
        prepare_result = self._prepareMacroObj(macro_obj, params)
        return macro_obj, prepare_result

    def _createMacroObj(self, macro_name_or_meta, pars, init_opts={}):
        macro_meta = macro_name_or_meta
        if isinstance(macro_meta, (str, unicode)):
            macro_meta = self.macro_manager.getMacro(macro_meta)

        macro_opts = {
            'executor'    : self,
            'environment' : self.macro_server
        }
        macro_opts.update(init_opts)
        if not macro_opts.has_key('id'):
            macro_opts['id'] = str(self.getNewMacroID())

        macroObj = self.macro_manager.createMacroObjFromMeta(macro_meta, pars,
                                                             init_opts=macro_opts)

        return macroObj

    def _prepareMacroObj(self, macro_obj, pars, prepare_opts={}):
        return self.macro_manager.prepareMacroObj(macro_obj, pars,
                                                  prepare_opts=prepare_opts)

    def prepareMacroObj(self, macro_name_or_meta, pars, init_opts={},
                        prepare_opts={}):
        """Prepare a new macro for execution

        :param macro_name_or_meta name: name of the macro to be prepared or
                                        the macro meta itself
        :param pars: list of parameter objects
        :param init_opts: keyword parameters for the macro constructor
        :param prepare_opts: keyword parameters for the macro prepare

        :return: a tuple of two elements: macro object, the result of preparing the macro"""
        macroObj = self._createMacroObj(macro_name_or_meta, pars, init_opts=init_opts)
        prepare_result = self._prepareMacroObj(macroObj, pars, prepare_opts=prepare_opts)
        return macroObj, prepare_result

    def prepareMacro(self, pars, init_opts={}, prepare_opts={}):
        """Prepare a new macro for execution
           Several different parameter formats are supported:
           1. several parameters:
             1.1 executor.prepareMacro('ascan', 'th', '0', '100', '10', '1.0')
             1.2 executor.prepareMacro('ascan', 'th', 0, 100, 10, 1.0)
             1.3 th = self.getObj('th');
                 executor.prepareMacro('ascan', th, 0, 100, 10, 1.0)
           2. a sequence of parameters:
              2.1 executor.prepareMacro(['ascan', 'th', '0', '100', '10', '1.0')
              2.2 executor.prepareMacro(('ascan', 'th', 0, 100, 10, 1.0))
              2.3 th = self.getObj('th');
                  executor.prepareMacro(['ascan', th, 0, 100, 10, 1.0])
           3. a space separated string of parameters:
              executor.prepareMacro('ascan th 0 100 10 1.0')

        :param pars: the command parameters as explained above
        :param opts: keyword optional parameters for prepare
        :return: a tuple of two elements: macro object, the result of preparing the macro
        """
        par0 = pars[0]
        if len(pars) == 1:
            if is_pure_str(par0):
                pars = par0.split(' ')
            elif is_non_str_seq(par0):
                pars = par0
        pars = map(str, pars)

        macro_klass, str_pars, pars = self._decodeMacroParameters(pars)

        init_opts['macro_line'] = "%s(%s) -> [%s]" % (str_pars[0], ", ".join(str_pars[1:]), id)
        if not init_opts.has_key('id'):
            init_opts['id'] = str(self.getNewMacroID())
        return self.prepareMacroObj(macro_klass, pars, init_opts, prepare_opts)

    def getRunningMacro(self):
        return self._macro_pointer

    def __stopObjects(self):
        """Stops all the reserved objects in the executor"""
        for _, objs in self._reserved_macro_objs.items():
            for obj in objs:
                try:
                    obj.stop()
                except AttributeError:
                    pass
                except:
                    self.warning("Unable to stop %s" % obj)
                    self.debug("Details:", exc_info=1)

    def __abortObjects(self):
        """Aborts all the reserved objects in the executor"""
        for _, objs in self._reserved_macro_objs.items():
            for obj in objs:
                try:
                    obj.abort()
                except AttributeError:
                    pass
                except:
                    self.warning("Unable to abort %s" % obj)
                    self.debug("Details:", exc_info=1)

    def abort(self):
        self.macro_server.add_job(self._abort, None)

    def stop(self):
        self.macro_server.add_job(self._stop, None)

    def _abort(self):
        m = self.getRunningMacro()
        if m is not None:
            self._aborted = True
            m.abort()
        self.__abortObjects()

    def _stop(self):
        m = self.getRunningMacro()
        if m is not None:
            self._stopped = True
            m.stop()
            if m.isPaused():
                m.resume(cb=self._macroResumed)
        self.__stopObjects()

    def pause(self):
        self._paused = True
        m = self.getRunningMacro()
        if m is not None:
            m.pause(cb=self._macroPaused)

    def _macroPaused(self, m):
        """Calback that is executed when the macro has efectively paused"""
        self.sendMacroStatusPause()
        self.sendState(Macro.Pause)

    def resume(self):
        if not self._paused:
            return
        self._paused = False
        m = self.getRunningMacro()
        if m is not None:
            m.resume(cb=self._macroResumed)

    def _macroResumed(self, m):
        """Callback that is executed when the macro has effectively resumed
        execution after being paused"""
        self.sendMacroStatusResume()
        self.sendState(Macro.Running)

    def run(self, params, asynch=True):
        """Runs the given macro(s)

        :param params: (sequence<str>) can be either a sequence of <macro name> [, <macro_parameter> ]
                       or a sequence with a single element which represents the xml string for a
                       macro script
        :return: (lxml.etree.Element) the xml representation of the running macro
        """
        # dict<PoolElement, set<Macro>>
        # key PoolElement - reserved object
        # value set<Macro> macros that reserved the object
        self._reserved_objs = {}

        # dict<Macro, seq<PoolElement>>
        # key Macro - macro object
        # value - sequence of reserved objects by the macro
        self._reserved_macro_objs = {}

        # reset the stacks
        self._macro_stack = []
        self._xml_stack = []
        self._macro_pointer = None
        self._aborted = False
        self._stopped = False
        self._paused = False
        self._last_macro_status = None

        # convert given parameters into an xml
        self._xml = self._preprocessParameters(params)

        if asynch:
            # start the job of actually running the macro
            self.macro_server.add_job(self.__runXML, self._jobEnded)
            #return the proper xml
            return self._xml
        else:
            self.__runXML()
            #return self._macro_pointer.getResult()

    def _jobEnded(self, *args, **kw):
        self.debug("Job ended (stopped=%s, aborted=%s)",
                   self._stopped, self._aborted)

    def __runXML(self):
        self.sendState(Macro.Running)
        try:
            self.__runStatelessXML()
            self.sendState(Macro.Finished)
        except AbortException:
            self.sendState(Macro.Abort)
        except Exception:
            self.sendState(Macro.Abort)
        finally:
            self._macro_stack = None
            self._xml_stack = None

    def __runStatelessXML(self, xml=None):
        if xml is None:
            xml = self._xml
        node = xml.tag
        if node == 'sequence':
            for xml_macro in xml.findall('macro'):
                self.__runXMLMacro(xml_macro)
        elif node == 'macro':
            self.__runXMLMacro(xml)

    def __runXMLMacro(self, xml):
        assert xml.tag == 'macro'
        try:
            macro_obj, _ = self._prepareXMLMacro(xml)
        except AbortException as ae:
            raise ae
        except Exception as e:
            door = self.door
            door.error("Error: %s", str(e))
            door.debug("Error details:", exc_info=1)
            raise e

        self._xml_stack.append(xml)
        try:
            self.runMacro(macro_obj)
        finally:
            self._xml_stack.pop()

    _runXMLMacro = __runXMLMacro

    def runMacro(self, macro_obj):
        name = macro_obj._getName()
        desc = macro_obj._getDescription()
        door = self.door
        if self._aborted:
            self.sendMacroStatusAbort()
            raise AbortException("aborted between macros (before %s)" % name)
        elif self._stopped:
            self.sendMacroStatusStop()
            raise StopException("stopped between macros (before %s)" % name)
        macro_exp, tb, result = None, None, None
        try:
            self.debug("[START] runMacro %s" % desc)
            self._macro_pointer = macro_obj
            self._macro_stack.append(macro_obj)
            for step in macro_obj.exec_():
                self.sendMacroStatus((step,))

            if macro_obj.hasResult() and macro_obj.getParentMacro() is None:
                result = self.__preprocessResult(macro_obj.getResult())
                door.debug("sending result %s", result)
                self.sendResult(result)
        except AbortException as ae:
            macro_exp = ae
        except StopException as se:
            macro_exp = se
        except MacroServerException as mse:
            exc_info = sys.exc_info()
            macro_exp = mse
            if not mse.traceback:
                mse.traceback = traceback.format_exc()
        except DevFailed as df:
            exc_info = sys.exc_info()
            exp_pars = {'type'      : df[0].reason,
                        'msg'       : df[0].desc,
                        'args'      : df.args,
                        'traceback' : traceback.format_exc() }
            macro_exp = MacroServerException(exp_pars)
        except Exception, err:
            exc_info = sys.exc_info()
            exp_pars = {'type'      : err.__class__.__name__,
                        'msg'       : err.args[0],
                        'args'      : err.args,
                        'traceback' : traceback.format_exc() }
            macro_exp = MacroServerException(exp_pars)
        finally:
            self.returnObjs(self._macro_pointer)

        # make sure the macro's on_abort is called and that a proper macro
        # status is sent
        if self._stopped:
            macro_obj._stopOnError()
            self.sendMacroStatusStop()
        elif self._aborted:
            macro_obj._abortOnError()
            self.sendMacroStatusAbort()

        # From this point on don't call any method of macro_obj which is part
        # of the mAPI (methods decorated with @mAPI) to avoid throwing an
        # AbortException if an Abort has been performed.
        if macro_exp is not None:
            if not self._stopped and not self._aborted:
                self.sendMacroStatusException(exc_info)
            self.debug("[ENDEX] (%s) runMacro %s" % (macro_exp.__class__.__name__, name))
            if isinstance(macro_exp, MacroServerException) and macro_obj.parent_macro is None:
                door.debug(macro_exp.traceback)
                door.error("An error occurred while running %s:\n%s" % (macro_obj.description, macro_exp.msg))
            self._popMacro()
            raise macro_exp
        self.debug("[ END ] runMacro %s" % desc)
        self._popMacro()
        return result

    def _popMacro(self):
        self._macro_stack.pop()
        length = len(self._macro_stack)
        if length > 0:
            self._macro_pointer = self._macro_stack[-1]
        #disable macro data for now. Comment following lines to enable it again
        else:
            self._macro_pointer = None

    def sendState(self, state):
        return self.door.set_state(state)

    def sendStatus(self, status):
        return self.door.set_status(status)

    def sendResult(self, result):
        return self.door.set_result(result)

    def getLastMacroStatus(self):
        return self._macro_pointer._getMacroStatus()

    def sendMacroStatusFinish(self):
        ms = self.getLastMacroStatus()
        if ms is not None:
            ms['state'] = 'finish'

            self.debug("Sending finish event")
            self.sendMacroStatus((ms,))

    def sendMacroStatusStop(self):
        ms = self.getLastMacroStatus()
        if ms is not None:
            ms['state'] = 'stop'

            self.debug("Sending stop event")
            self.sendMacroStatus((ms,))

    def sendMacroStatusAbort(self):
        ms = self.getLastMacroStatus()
        if ms is not None:
            ms['state'] = 'abort'

            self.debug("Sending abort event")
            self.sendMacroStatus((ms,))

    def sendMacroStatusException(self, exc_info):
        ms = self.getLastMacroStatus()
        if ms is not None:
            ms['state'] = 'exception'
            ms['exc_type'] = str(exc_info[0])
            ms['exc_value'] = str(exc_info[1])
            ms['exc_stack'] = traceback.format_exception(*exc_info)
            self.debug("Sending exception event")
            self.sendMacroStatus((ms,))

    def sendMacroStatusPause(self):
        ms = self.getLastMacroStatus()
        if ms is not None and len(ms) > 0:
            ms['state'] = 'pause'
            self.debug("Sending pause event")
            self.sendMacroStatus((ms,))

    def sendMacroStatusResume(self):
        ms = self.getLastMacroStatus()
        if ms is not None and len(ms) > 0:
            ms['state'] = 'resume'
            self.debug("Sending resume event")
            self.sendMacroStatus((ms,))

    def sendMacroStatus(self, data):
        self._last_macro_status = data
        #data = self._macro_status_codec.encode(('', data))
        return self.door.set_macro_status(data)

    def sendRecordData(self, data, codec=None):
        return self.door.set_record_data(data, codec=codec)

    def reserveObj(self, obj, macro_obj, priority=0):
        if obj is None or macro_obj is None: return

        # Fill _reserved_macro_objs
        objs = self._reserved_macro_objs[macro_obj] = \
            self._reserved_macro_objs.get(macro_obj, list())
        if not obj in objs:
            if priority:
                objs.insert(0, obj)
            else:
                objs.append(obj)

        # Fill _reserved_objs
        macros = self._reserved_objs[obj] = self._reserved_objs.get(obj, set())
        macros.add(macro_obj)

        # Tell the object that it is reserved by a new macro
        if hasattr(obj, 'reserve'): obj.reserve(macro_obj)

    def returnObjs(self, macro_obj):
        """Free the macro reserved objects"""
        if macro_obj is None: return
        objs = self._reserved_macro_objs.get(macro_obj)
        if objs is None: return

        # inside returnObj we change the list so we have to iterate with a copy
        for obj in copy.copy(objs):
            self.returnObj(obj, macro_obj)

    def returnObj(self, obj, macro_obj):
        """Free an object reserved by a macro"""
        if obj is None or macro_obj is None: return

        if hasattr(obj, 'unreserve'): obj.unreserve()
        objs = self._reserved_macro_objs.get(macro_obj)
        if objs is None:
            return
        objs.remove(obj)
        if len(objs) == 0:
            del self._reserved_macro_objs[macro_obj]

        try:
            macros = self._reserved_objs[obj]
            macros.remove(macro_obj)
            if not len(macros):
                del self._reserved_objs[obj]
        except KeyError:
            self.debug("Unexpected KeyError trying to remove reserved object")



#!/usr/bin/env python

##############################################################################
##
## This file is part of Sardana
##
## http://www.tango-controls.org/static/sardana/latest/doc/html/index.html
##
## Copyright 2011 CELLS / ALBA Synchrotron, Bellaterra, Spain
##
## Sardana is free software: you can redistribute it and/or modify
## it under the terms of the GNU Lesser General Public License as published by
## the Free Software Foundation, either version 3 of the License, or
## (at your option) any later version.
##
## Sardana is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU Lesser General Public License for more details.
##
## You should have received a copy of the GNU Lesser General Public License
## along with Sardana.  If not, see <http://www.gnu.org/licenses/>.
##
##############################################################################

"""This module is part of the Python Pool libray. It defines the class for an
acquisition"""

__all__ = [ "AcquisitionState", "AcquisitionMap", "PoolCTAcquisition",
            "Pool0DAcquisition", "Channel", "PoolIORAcquisition" ]

__docformat__ = 'restructuredtext'

import time

from taurus.core.util import Enumeration, DebugIt

from sardana import State, ElementType, TYPE_TIMERABLE_ELEMENTS
from sardana.sardanathreadpool import get_thread_pool
from poolaction import ActionContext, PoolActionItem, PoolAction

#: enumeration representing possible motion states
AcquisitionState = Enumeration("AcquisitionState", ( \
    "Stopped",
#    "StoppedOnError",
#    "StoppedOnAbort",
    "Acquiring",
    "Invalid") )

AS = AcquisitionState
AcquiringStates = AS.Acquiring,
StoppedStates = AS.Stopped, #MS.StoppedOnError, MS.StoppedOnAbort

AcquisitionMap = {
    #AS.Stopped           : State.On,
    AS.Acquiring         : State.Moving,
    AS.Invalid           : State.Invalid,
}

class PoolAcquisition(PoolAction):

    def __init__(self, main_element, name="Acquisition"):
        PoolAction.__init__(self, main_element, name)
        ctname = name + ".CTAcquisition"
        zerodname = name + ".0DAcquisition"
        self._0d_acq = zd_acq = Pool0DAcquisition(main_element, name=zerodname)
        self._ct_acq = PoolCTAcquisition(main_element, name=ctname, slaves=(zd_acq,))

    def run(self, *args, **kwargs):
        n = kwargs.get('multiple', 1)
        if n == 1:
            return self._run_single(*args, **kwargs)
        return self._run_multiple(*args, **kwargs)
    
    def _run_multiple(self, *args, **kwargs):
        n = kwargs['multiple']
        synch = kwargs.get("synch", False)
        if synch:
            for _ in range(n):
                self._run_single(self, *args, **kwargs)
        else:
            kwargs["synch"] = True
            get_thread_pool().add(self._run_multiple, None, *args, **kwargs)
    
    def _run_single(self, *args, **kwargs):
        """Runs this action"""
        synch = kwargs.get("synch", False)
        ct_acq = self._ct_acq
        zd_acq = self._0d_acq
        if synch:
            ct_acq.run(*args, **kwargs)
        else:
            ct_acq.run(*args, **kwargs)
            zd_acq.run(*args, **kwargs)

    def _get_acq_for_element(self, element):
        elem_type = element.get_type()
        if elem_type in TYPE_TIMERABLE_ELEMENTS:
            return self._ct_acq
        elif elem_type == ElementType.ZeroDExpChannel:
            return self._0d_acq

    def clear_elements(self):
        """Clears all elements from this action"""

    def add_element(self, element):
        """Adds a new element to this action.

        :param element: the new element to be added
        :type element: sardana.pool.poolelement.PoolElement"""
        return self._get_acq_for_element(element).add_element(element)

    def remove_element(self, element):
        """Removes an element from this action. If the element is not part of
        this action, a ValueError is raised.

        :param element: the new element to be removed
        :type element: sardana.pool.poolelement.PoolElement

        :raises: ValueError"""
        return self._get_acq_for_element(element).add_element(element)

    def get_elements(self, copy_of=False):
        """Returns a sequence of all elements involved in this action.

        :param copy_of: If False (default) the internal container of elements is
                        returned. If True, a copy of the internal container is
                        returned instead
        :type copy_of: bool
        :return: a sequence of all elements involved in this action.
        :rtype: seq<sardana.pool.poolelement.PoolElement>"""
        return self._ct_acq.get_elements() + self._0d_acq.get_elements()

    def get_pool_controller_list(self):
        """Returns a list of all controller elements involved in this action.

        :return: a list of all controller elements involved in this action.
        :rtype: list<sardana.pool.poolelement.PoolController>"""
        return self._pool_ctrl_list

    def get_pool_controllers(self):
        """Returns a dict of all controller elements involved in this action.

        :return: a dict of all controller elements involved in this action.
        :rtype: dict<sardana.pool.poolelement.PoolController, seq<sardana.pool.poolelement.PoolElement>>"""
        ret = {}
        ret.update(self._ct_acq.get_pool_controllers())
        ret.update(self._0d_acq.get_pool_controllers())
        return ret

    def read_value(self, ret=None, serial=False):
        """Reads value information of all elements involved in this action

        :param ret: output map parameter that should be filled with value
                    information. If None is given (default), a new map is
                    created an returned
        :type ret: dict
        :param serial: If False (default) perform controller HW value requests
                       in parallel. If True, access is serialized.
        :type serial: bool
        :return: a map containing value information per element
        :rtype: dict<:class:~`sardana.pool.poolelement.PoolElement`,
                     :class:~`sardana.sardanavalue.SardanaValue`>"""
        ret = self._ct_acq.read_value(ret=ret, serial=serial)
        ret.update(self._0d_acq.read_value(ret=ret, serial=serial))
        return ret


class Channel(PoolActionItem):

    def __init__(self, acquirable, info=None):
        PoolActionItem.__init__(self, acquirable)
        if info:
            self.__dict__.update(info)

    def __getattr__(self, name):
        return getattr(self.element, name)


class PoolCTAcquisition(PoolAction):

    def __init__(self, main_element, name="CTAcquisition", slaves=None):
        self._channels = None

        if slaves is None:
            slaves = ()
        self._slaves = slaves

        PoolAction.__init__(self, main_element, name)

    def get_read_value_loop_ctrls(self):
        return self._pool_ctrl_dict_loop
    
    def start_action(self, *args, **kwargs):
        """Prepares everything for acquisition and starts it.

           :param: config"""

        pool = self.pool

        # prepare data structures
        self._aborted = False
        self._stopped = False

        self._acq_sleep_time = kwargs.pop("acq_sleep_time",
                                             pool.acq_loop_sleep_time)
        self._nb_states_per_value = \
            kwargs.pop("nb_states_per_value",
                       pool.acq_loop_states_per_value)

        self._integ_time = integ_time = kwargs.get("integ_time")
        self._mon_count = mon_count = kwargs.get("monitor_count")
        if integ_time is None and mon_count is None:
            raise Exception("must give integration time or monitor counts")
        if integ_time is not None and mon_count is not None:
            raise Exception("must give either integration time or monitor counts (not both)")

        _ = kwargs.get("items", self.get_elements())
        cfg = kwargs['config']

        # determine which is the controller which holds the master channel
        
        if integ_time is not None:
            master_key = 'timer'
            master_value = integ_time
        if mon_count is not None:
            master_key = 'monitor'
            master_value = - mon_count

        master = cfg[master_key]
        master_ctrl = master.controller

        pool_ctrls_dict = dict(cfg['controllers'])
        pool_ctrls_dict.pop('__tango__', None)
        pool_ctrls = []
        self._pool_ctrl_dict_loop = _pool_ctrl_dict_loop = {}
        for ctrl, v in pool_ctrls_dict.items():
            if ctrl.is_timerable():
                pool_ctrls.append(ctrl)
            if ElementType.CTExpChannel in ctrl.get_ctrl_types():
                _pool_ctrl_dict_loop[ctrl] = v
            
        # make sure the controller which has the master channel is the last to
        # be called
        pool_ctrls.remove(master_ctrl)
        pool_ctrls.append(master_ctrl)

        # Determine which channels are active
        self._channels = channels = {}
        for pool_ctrl in pool_ctrls:
            ctrl = pool_ctrl.ctrl
            pool_ctrl_data = pool_ctrls_dict[pool_ctrl]
            main_unit_data = pool_ctrl_data['units']['0']
            elements = main_unit_data['channels']

            for element, element_info in elements.items():
                axis = element.axis
                channel = Channel(element, info=element_info)
                channels[element] = channel

        #for channel in channels:
        #    channel.prepare_to_acquire(self)

        with ActionContext(self):

            # PreLoadAll, PreLoadOne, LoadOne and LoadAll
            for pool_ctrl in pool_ctrls:
                ctrl = pool_ctrl.ctrl
                pool_ctrl_data = pool_ctrls_dict[pool_ctrl]
                main_unit_data = pool_ctrl_data['units']['0']
                ctrl.PreLoadAll()
                master = main_unit_data[master_key]
                axis = master.axis
                res = ctrl.PreLoadOne(axis, master_value)
                if not res:
                    raise Exception("%s.PreLoadOne(%d) returns False" % (pool_ctrl.name, axis,))
                ctrl.LoadOne(axis, master_value)
                ctrl.LoadAll()

            # PreStartAll on all controllers
            for pool_ctrl in pool_ctrls:
                pool_ctrl.ctrl.PreStartAll()

            # PreStartOne & StartOne on all elements
            for pool_ctrl in pool_ctrls:
                ctrl = pool_ctrl.ctrl
                pool_ctrl_data = pool_ctrls_dict[pool_ctrl]
                main_unit_data = pool_ctrl_data['units']['0']
                elements = main_unit_data['channels']
                for element in elements:
                    axis = element.axis
                    channel = channels[element]
                    if channel.enabled:
                        ret = ctrl.PreStartOne(axis, master_value)
                        if not ret:
                            raise Exception("%s.PreStartOne(%d) returns False" \
                                            % (pool_ctrl.name, axis))
                        ctrl.StartOne(axis, master_value)

            # set the state of all elements to  and inform their listeners
            for channel in channels:
                channel.set_state(State.Moving, propagate=2)

            # StartAll on all controllers
            for pool_ctrl in pool_ctrls:
                pool_ctrl.ctrl.StartAll()

    def in_acquisition(self, states):
        """Determines if we are in acquisition or if the acquisition has ended
        based on the current unit trigger modes and states returned by the
        controller(s)

        :param states: a map containing state information as returned by
                       read_state_info
        :type states: dict<PoolElement, State>
        :return: returns True if in acquisition or False otherwise
        :rtype: bool"""
        for elem in states:
            s = states[elem][0][0]
            if self._is_in_action(s):
                return True

    def action_loop(self):
        i = 0

        states, values = {}, {}
        for element in self._channels:
            states[element] = None
            #values[element] = None

        nap = self._acq_sleep_time
        nb_states_per_value = self._nb_states_per_value

        # read values to send a first event when starting to acquire
        with ActionContext(self):
            self.raw_read_value_loop(ret=values)
            for acquirable, value in values.items():
                acquirable.put_value(value, propagate=2)
        
        while True:
            self.read_state_info(ret=states)

            if not self.in_acquisition(states):
                break

            # read value every n times
            if not i % nb_states_per_value:
                self.read_value_loop(ret=values)
                for acquirable, value in values.items():
                    acquirable.put_value(value)
                    
            time.sleep(nap)
            i += 1

        for slave in self._slaves:
            try:
                slave.stop_action()
            except:
                self.warning("Unable to stop slave acquisition %s",
                             slave.getLogName())
                self.debug("Details", exc_info=1)

        with ActionContext(self):
            self.raw_read_state_info(ret=states)
            self.raw_read_value_loop(ret=values)

        for acquirable, state_info in states.items():
            # first update the element state so that value calculation
            # that is done after takes the updated state into account
            acquirable.set_state_info(state_info, propagate=0)
            if acquirable in values:
                value = values[acquirable]
                acquirable.put_value(value, propagate=2)
            with acquirable:
                acquirable.clear_operation()
                state_info = acquirable._from_ctrl_state_info(state_info)
                acquirable.set_state_info(state_info, propagate=2)


class Pool0DAcquisition(PoolAction):

    def __init__(self, main_element, name="0DAcquisition"):
        self._channels = None
        PoolAction.__init__(self, main_element, name)

    def start_action(self, *args, **kwargs):
        """Prepares everything for acquisition and starts it.

           :param: config"""

        pool = self.pool

        # prepare data structures
        self._aborted = False
        self._stopped = False

        self._acq_sleep_time = kwargs.pop("acq_sleep_time",
                                          pool.acq_loop_sleep_time)
        self._nb_states_per_value = \
            kwargs.pop("nb_states_per_value",
                       pool.acq_loop_states_per_value)

        integ_time = kwargs.get("integ_time")
        mon_count = kwargs.get("monitor_count")
        if integ_time is None and mon_count is None:
            raise Exception("must give integration time or monitor counts")
        if integ_time is not None and mon_count is not None:
            raise Exception("must give either integration time or monitor counts (not both)")

        items = kwargs.get("items")
        if items is None:
            items = self.get_elements()
        cfg = kwargs['config']

        pool_ctrls_dict = dict(cfg['controllers'])
        pool_ctrls_dict.pop('__tango__', None)
        pool_ctrls = []
        for ctrl in pool_ctrls_dict:
            if ElementType.ZeroDExpChannel in ctrl.get_ctrl_types():
                pool_ctrls.append(ctrl)

        # Determine which channels are active
        self._channels = channels = {}
        for pool_ctrl in pool_ctrls:
            ctrl = pool_ctrl.ctrl
            pool_ctrl_data = pool_ctrls_dict[pool_ctrl]
            main_unit_data = pool_ctrl_data['units']['0']
            elements = main_unit_data['channels']

            for element, element_info in elements.items():
                channel = Channel(element, info=element_info)
                channels[element] = channel

        with ActionContext(self):
            # set the state of all elements to  and inform their listeners
            for channel in channels:
                channel.clear_buffer()
                channel.set_state(State.Moving, propagate=2)

    def in_acquisition(self, states):
        """Determines if we are in acquisition or if the acquisition has ended
        based on the current unit trigger modes and states returned by the
        controller(s)

        :param states: a map containing state information as returned by
                       read_state_info
        :type states: dict<PoolElement, State>
        :return: returns True if in acquisition or False otherwise
        :rtype: bool"""
        for state in states:
            s = states[state][0]
            if self._is_in_action(s):
                return True

    def action_loop(self):
        i = 0

        states, values = {}, {}
        for element in self._channels:
            states[element] = None
            values[element] = None

        nap = self._acq_sleep_time
        while not (self._stopped or self._aborted):
            self.read_value(ret=values)
            for acquirable, value in values.items():
                acquirable.put_value(value)

            i += 1
            time.sleep(nap)

        with ActionContext(self):
            self.raw_read_state_info(ret=states)

        for acquirable, state_info in states.items():
            # first update the element state so that value calculation
            # that is done after takes the updated state into account
            state_info = acquirable._from_ctrl_state_info(state_info)
            acquirable.set_state_info(state_info, propagate=0)
            with acquirable:
                acquirable.clear_operation()
                acquirable.set_state_info(state_info, propagate=2)

    def stop_action(self, *args, **kwargs):
        """Stop procedure for this action."""
        self._stopped = True

    def abort_action(self, *args, **kwargs):
        """Aborts procedure for this action"""
        self._aborted = True


class PoolIORAcquisition(PoolAction):

    def __init__(self, pool, name="IORAcquisition"):
        self._channels = None
        PoolAction.__init__(self, pool, name)

    def start_action(self, *args, **kwargs):
        pass

    def in_acquisition(self, states):
        return True
        pass

    @DebugIt()
    def action_loop(self):
        i = 0

        states, values = {}, {}
        for element in self._channels:
            states[element] = None
            values[element] = None

        # read values to send a first event when starting to acquire
        self.read_value(ret=values)
        for acquirable, value in values.items():
            acquirable.put_value(value, propagate=2)

        while True:
            self.read_state_info(ret=states)

            if not self.in_acquisition(states):
                break

            # read value every n times
            if not i % 5:
                self.read_value(ret=values)
                for acquirable, value in values.items():
                    acquirable.put_value(value)

            i += 1
            time.sleep(0.01)

        self.read_state_info(ret=states)

        # first update the element state so that value calculation
        # that is done after takes the updated state into account
        for acquirable, state_info in states.items():
            acquirable.set_state_info(state_info, propagate=0)

        # Do NOT send events before we exit the OperationContext, otherwise
        # we may be asked to start another action before we leave the context
        # of the current action. Instead, send the events in the finish hook
        # which is executed outside the OperationContext

        def finish_hook(*args, **kwargs):
            # read values and propagate the change to all listeners
            self.read_value(ret=values)
            for acquirable, value in values.items():
                acquirable.put_value(value, propagate=2)

            # finally set the state and propagate to all listeners
            for acquirable, state_info in states.items():
                acquirable.set_state_info(state_info, propagate=2)

        self.set_finish_hook(finish_hook)



##############################################################################
##
## This file is part of Sardana
##
## http://www.tango-controls.org/static/sardana/latest/doc/html/index.html
##
## Copyright 2011 CELLS / ALBA Synchrotron, Bellaterra, Spain
## 
## Sardana is free software: you can redistribute it and/or modify
## it under the terms of the GNU Lesser General Public License as published by
## the Free Software Foundation, either version 3 of the License, or
## (at your option) any later version.
## 
## Sardana is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU Lesser General Public License for more details.
## 
## You should have received a copy of the GNU Lesser General Public License
## along with Sardana.  If not, see <http://www.gnu.org/licenses/>.
##
##############################################################################

"""
A macro package to show examples on how to run a macro from inside another macro
"""

__all__ = ["call_wa", "call_wm", "subsubm", "subm", "mainmacro", "runsubs"]

__docformat__ = 'restructuredtext'

from sardana.macroserver.macro import Macro, Type, ParamRepeat

#-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-
# First example:
# A 'mainmacro' that executes a 'subm' that in turn executes a 'subsubm'.
# The 'subsubm' macro itself calls a short ascan macro
#-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~--~-~-

class call_wa(Macro):
    
    def run(self):
        self.macros.wa()
        
class call_wm(Macro):

    param_def = [
        ['motor_list',
         ParamRepeat(['motor', Type.Motor, None, 'Motor to move']),
         None, 'List of motor to show'],
    ]

    def run(self, *m):
        self.macros.wm(*m)
        
class subsubm(Macro):
    """this macro just calls the 'subm' macro
    This macro is part of the examples package. It was written for demonstration purposes"""
    def run(self):
        self.output("Starting %s" % self.getName())
        m = self.macros
        motors = self.getObjs('.*', type_class=Type.Motor)
        m.ascan(motors[0], 0, 100, 10, 0.2)
        self.output("Finished %s" % self.getName())

class subm(Macro):
    """this macro just calls the 'subsubm' macro
    This macro is part of the examples package. It was written for demonstration purposes"""
    
    def run(self):
        self.output("Starting %s" % self.getName())
        self.macros.subsubm()
        self.output("Finished %s" % self.getName())

class mainmacro(Macro):
    """this macro just calls the 'subm' macro
    This macro is part of the examples package. It was written for demonstration purposes"""
    
    def run(self):
        self.output("Starting %s" % self.getName())
        self.macros.subm()
        self.output("Finished %s" % self.getName())

#-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-
# Second example:
# a 'runsubs' macro that shows the different ways to call a macro from inside
# another macro
#-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~--~-~-

class runsubs(Macro):
    """ A macro that calls a ascan macro using the motor given as first parameter.
    
    This macro is part of the examples package. It was written for demonstration purposes
    
    Call type will allow to choose to format in which the ascan macro is called
    from this macro:
    1 - m.ascan(motor.getName(), '0', '10', '4', '0.2')
    2 - m.ascan(motor, 0, 10, 4, 0.2)
    3 - self.execMacro('ascan', motor.getName(), '0', '10', '4', '0.2')
    4 - self.execMacro(['ascan', motor, 0, 10, 4, 0.2])
    5 - params = 'ascan', motor, 0, 10, 4, 0.2
        self.execMacro(params)
    6 - self.execMacro("ascan %s 0 10 4 0.2" % motor.getName())
    7 - macro, prep = self.createMacro("ascan %s 0 10 4 0.2" % motor.getName())
        macro.hooks = [ self.hook ]
        self.runMacro(macro)
    8 - macro, prep = self.createMacro('ascan', motor, 0, 10, 4, 0.2)
        macro.hooks = [ self.hook ]
        self.runMacro(macro)
    9 - params = 'ascan', motor, 0, 10, 4, 0.2
        macro, prep = self.createMacro(params)
        macro.hooks = [ self.hook ]
        self.runMacro(macro)
        
        Options 7,8 and 9 use the lower level macro API in order to be able to
        attach hooks to the ascan macro."""
    param_def = [
       ['motor',      Type.Motor,   None, 'Motor to move'],
       ['call_type',  Type.Integer, 2, 'type of run to execute internally'],
    ]
    
    def hook(self):
        self.info("executing hook in a step of a scan...")
    
    def run(self, motor, call_type):
        m = self.macros
        self.output("Using type %d" % call_type)
        if call_type == 1:
            m.ascan(motor.getName(), '0', '10', '4', '0.2')
        elif call_type == 2:
            m.ascan(motor, 0, 10, 4, 0.2)
        elif call_type == 3:
            self.execMacro('ascan', motor.getName(), '0', '10', '4', '0.2')
        elif call_type == 4:
            self.execMacro('ascan', motor, 0, 10, 4, 0.2)
        elif call_type == 5:
            params = 'ascan', motor, 0, 10, 4, 0.2
            self.execMacro(params)
        elif call_type == 6:
            self.execMacro("ascan %s 0 10 4 0.2" % motor.getName())
        elif call_type == 7:
            macro, prep = self.createMacro("ascan %s 0 10 4 0.2" % \
                                                     motor.getName())
            macro.hooks = [ self.hook ]
            self.runMacro(macro)
        elif call_type == 8:
            macro, prep = self.createMacro('ascan', motor, 0, 10, 4, 0.2)
            macro.hooks = [ self.hook ]
            self.runMacro(macro)
        elif call_type == 9:
            params = 'ascan', motor, 0, 10, 4, 0.2
            macro, prep = self.createMacro(params)
            macro.hooks = [ self.hook ]
            self.runMacro(macro)
        
            



##############################################################################
##
## This file is part of Sardana
##
## http://www.tango-controls.org/static/sardana/latest/doc/html/index.html
##
## Copyright 2011 CELLS / ALBA Synchrotron, Bellaterra, Spain
## 
## Sardana is free software: you can redistribute it and/or modify
## it under the terms of the GNU Lesser General Public License as published by
## the Free Software Foundation, either version 3 of the License, or
## (at your option) any later version.
## 
## Sardana is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU Lesser General Public License for more details.
## 
## You should have received a copy of the GNU Lesser General Public License
## along with Sardana.  If not, see <http://www.gnu.org/licenses/>.
##
##############################################################################

import random

from sardana import State
from sardana.pool.controller import ZeroDController

class Channel:
    
    def __init__(self,idx):
        self.idx = idx            # 1 based index
        self.value = 0.0
        self.active = False


class DummyZeroDController(ZeroDController):
    """This class represents a dummy Sardana 0D controller."""

    gender = "Simulation"
    model  = "Basic"
    organization = "Sardana team"

    MaxDevice = 1024

    def __init__(self, inst, props, *args, **kwargs):
        ZeroDController.__init__(self, inst, props, *args, **kwargs)

        self.channels = [ Channel(i+1) for i in xrange(self.MaxDevice) ]
        self.read_channels = {}
        
    def AddDevice(self,ind):
        self.channels[ind].active = True
        
    def DeleteDevice(self,ind):
        self.channels[ind].active = False

    def StateOne(self,ind):
        return State.On, "OK"
        
    def _setChannelValue(self, channel):
        channel.value = 100 * channel.idx + 10*(random.random()-0.5)

    def PreReadAll(self):
        self.read_channels = {}

    def PreReadOne(self,ind):
        channel = self.channels[ind-1]
        self.read_channels[ind] = channel

    def ReadAll(self):
        for channel in self.read_channels.values():
            self._setChannelValue(channel)

    def ReadOne(self,ind):
        v = self.read_channels[ind].value
        return v



#!/usr/bin/env python

##############################################################################
##
## This file is part of Sardana
##
## http://www.tango-controls.org/static/sardana/latest/doc/html/index.html
##
## Copyright 2011 CELLS / ALBA Synchrotron, Bellaterra, Spain
## 
## Sardana is free software: you can redistribute it and/or modify
## it under the terms of the GNU Lesser General Public License as published by
## the Free Software Foundation, either version 3 of the License, or
## (at your option) any later version.
## 
## Sardana is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU Lesser General Public License for more details.
## 
## You should have received a copy of the GNU Lesser General Public License
## along with Sardana.  If not, see <http://www.gnu.org/licenses/>.
##
##############################################################################

"""This module is part of the Python Sardana library. It defines the base classes
for module manager"""

from __future__ import with_statement

__all__ = ["ModuleManager"]

__docformat__ = 'restructuredtext'

import sys
import imp
import threading

from taurus.core import ManagerState
from taurus.core.util import Singleton, Logger

from .sardanamanager import SardanaIDManager

_MINIMUM_EXCLUDE = [
'sys',
'os.path',
'__builtin__',
'__main__',
'sardana',
'taurus',
'PyTango'
]

class PathContext(object):
    
    def __init__(self, path):
        self.path = path
        
    def __enter__(self):
        self.orig_path = list(sys.path)
        if self.path is not None:
            sys.path = self.path + sys.path
    
    def __exit__(self, etype, evalue, etraceback):
        sys.path = self.orig_path

        
class PathManager(SardanaIDManager):
    
    def __init__(self):
        SardanaIDManager.__init__(self)
        self._orig_path = list(sys.path)
        self._path_lock = threading.Lock()
        #: dict<int(path_id), list<int(path start index), list<str(path)>>
        self._path_info = {}
    
    @staticmethod
    def _decode_path(path):
        p = []
        for item in path:
            p.extend(item.split(":"))
        return p
    
    def add_python_path(self, path):
        """Adds a new path to the python path.
        
        :param path:
            a sequence of strings each string may contain an absolute path or a
            list of ":" or "\n" separated absolute paths
        :type path: list<str>
        :return:
            a path id identifying the changes that were made to sys.path. This
            ID can be used in :meth:`remove_path` to remove only the added path
        :rtype: int"""
        path = self._decode_path(path)
        path_len = len(path)
        pif = self._path_info
        
        with self._path_lock:
            path_id = self.get_new_id()
            
            for _, p_info in pif.items():
                p_info[0] += path_len
            
            pif[path_id] = [0, path]
            sys.path = path + sys.path
        return path_id
    
    def remove_python_path(self, path_id):
        """Removes the path given by the path_id
        
        :param path_id:
            a path id identifying specific changes that were made to sys.path
        :type path_id: int"""
        pif = self._path_info
        start, path = pif[path_id]
        path_len = len(path)
        
        with self._path_lock:
            sys.path = sys.path[:start+1] + sys.path[start+path_len:]
            del pif[path_id]
    
    def reset_python_path(self):
        with self._path_lock:
            sys.path = list(self._orig_path)
            self._path_info = {}
        

class ModuleManager(Singleton, Logger):
    """This class handles python module loading/reloading and unloading."""
    
    def __init__(self):
        """ Initialization. Nothing to be done here for now."""
        pass

    def init(self, *args, **kwargs):
        """Singleton instance initialization."""
        name = self.__class__.__name__
        self._state = ManagerState.UNINITIALIZED
        self._path_manager = PathManager()
        self.call__init__(Logger, name)
        self.reInit()
    
    def reInit(self):
        if self._state == ManagerState.INITED:
            return
        
        # dict<str, module>
        # key   - module name (without path and without extension)
        # value - python module object reference
        self._modules = {}
        
        self._state = ManagerState.INITED
    
    def cleanUp(self):
        if self._state == ManagerState.CLEANED:
            return
        
        self.unloadModules()
        
        self._modules = None
        
        self._state = ManagerState.CLEANED
    
    def reset_python_path(self):
        return self._path_manager.reset_python_path()
    
    def remove_python_path(self, path_id):
        return self._path_manager.remove_python_path(path_id)
    
    def add_python_path(self, path):
        return self._path_manager.add_python_path(path)
    
    def findFullModuleName(self, module_name, path=None):
        mfile = None
        try:
            mfile, pathname, _ = imp.find_module(module_name, path)
        finally:
            if mfile is not None:
                mfile.close()
        return pathname
    
    def reloadModule(self, module_name, path=None, reload=True):
        """Loads/reloads the given module name"""
        
        if not reload:
            return self.loadModule(module_name, path=path)
        
        self.unloadModule(module_name)
        
        m, mfile = None, None
        try:
            mfile, pathname, desc = imp.find_module(module_name, path)
            self.info("(re)loading module %s...", module_name)
            m = imp.load_module(module_name, mfile, pathname, desc)
        except:
            self.error("Error (re)loading module %s", module_name)
            self.debug("Details:", exc_info=1)
            raise
        finally:
            if mfile is not None:
                mfile.close()
        
        if m is None:
            return
        
        self._modules[module_name] = m
        
        return m

    def deep_reload_module(self, module_name, path=None, exclude=None):

        if module_name in sys.modules:
            module = sys.modules[module_name]
        else:
            module = self.loadModule(module_name, path)
            
        excl = list(_MINIMUM_EXCLUDE)
        if exclude is not None:
            excl.extend(exclude)
        
        import sardana.util.deepreload
        with PathContext(path):
            try:
                self.info("deep reloading module %s...", module_name)
                sardana.util.deepreload.reload(module, excl)
            except:
                self.error("Error deep reloading module %s", module_name)
                self.debug("Details:", exc_info=1)
                raise
                
    def loadModule(self, module_name, path=None):
        """Loads the given module name. If the module has been already loaded
        into this python interpreter, nothing is done.
        
        :param module_name: the module to be loaded.
        :type module_name: str
        :param path: list of paths to look for modules [default: None]
        :type path: seq<str> or None
        :return: python module
        
        :raises: ImportError"""
        
        if module_name in sys.modules:
            return sys.modules[module_name]
        
        with PathContext(path):
            self.info("loading module %s...", module_name)
            try:
                module = __import__(module_name, globals(), locals(), [], -1)
            except:
                self.error("Error loading module %s", module_name)
                self.debug("Details:", exc_info=1)
                raise
        
        self._modules[module_name] = module
        
        return module
    
    def unloadModule(self, module_name):
        """Unloads the given module name"""
        if self._modules.has_key(module_name):
            self.debug("unloading module %s" % module_name)
            assert(sys.modules.has_key(module_name))
            self._modules.pop(module_name)
            del sys.modules[module_name]
    
    def unloadModules(self, module_list=None):
        """Unloads the given module name"""
        modules = module_list or self._modules.keys()
        for module in modules:
            self.unloadModule(module)
    
    def getModule(self, module_name):
        """Returns the module object for the given module name"""
        m = self._modules.get(module_name)
        if m is None:
            m = self.reloadModule(module_name)
        return m

    def getModuleNames(self):
        module_names = self._modules.keys()
        module_names.sort()
        return module_names
    



#!/usr/bin/env python

##############################################################################
##
## This file is part of Sardana
##
## http://www.tango-controls.org/static/sardana/latest/doc/html/index.html
##
## Copyright 2011 CELLS / ALBA Synchrotron, Bellaterra, Spain
## 
## Sardana is free software: you can redistribute it and/or modify
## it under the terms of the GNU Lesser General Public License as published by
## the Free Software Foundation, either version 3 of the License, or
## (at your option) any later version.
## 
## Sardana is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU Lesser General Public License for more details.
## 
## You should have received a copy of the GNU Lesser General Public License
## along with Sardana.  If not, see <http://www.gnu.org/licenses/>.
##
##############################################################################

"""This module is part of the Python Sardana library. It defines the base
classes for Sardana object"""

__all__ = ["SardanaBaseObject", "SardanaObjectID"]

__docformat__ = 'restructuredtext'

import weakref

from taurus.core.util import Logger

from sardanadefs import ElementType, Interface, InterfacesExpanded, InvalidId
from sardanaevent import EventGenerator, EventReceiver


class SardanaBaseObject(EventGenerator, EventReceiver, Logger):
    """The Sardana most abstract object. It contains only two members:
    
       - _manager : a weak reference to the manager (pool or ms) where it
         belongs
       - _name : the name
       - _full_name : the name (usually a tango device name, but can be
         anything else.)"""
    
    def __init__(self, **kwargs):
        EventGenerator.__init__(self)
        EventReceiver.__init__(self)
        self._type = kwargs.pop('elem_type')
        self._name = intern(kwargs.pop('name'))
        self._full_name = intern(kwargs.pop('full_name'))
        self._frontend = None
        Logger.__init__(self, self._name)
        self._manager = weakref.ref(kwargs.pop('manager'))
        self._parent = weakref.ref(kwargs.pop('parent', self.manager))
        
    def get_manager(self):
        """Return the :class:`sardana.Manager` which *owns* this sardana
        object.
        
        :return: the manager which *owns* this pool object.
        :rtype: :class:`sardana.Manager`"""
        return self._manager()
    
    def get_name(self):
        """Returns this sardana object name
        
        :return: this sardana object name
        :rtype: str"""
        return self._name

    def get_full_name(self):
        """Returns this sardana object full name
        
        :return: this sardana object full name
        :rtype: str"""
        return self._full_name
    
    def get_type(self):
        """Returns this sardana object type.
        
        :return: this sardana object type
        :rtype: :obj:`~sardana.sardanadefs.ElementType`"""
        return self._type
    
    def get_parent(self):
        """Returns this pool object parent.
        
        :return: this objects parent
        :rtype: :class:`~sardana.sardanabase.SardanaBaseObject`"""
        return self._parent()

    def get_parent_name(self):
        """Returns this sardana object parent's name.
        
        :return: this objects parent
        :rtype: str"""
        parent = self.get_parent()
        if parent and hasattr(parent, 'name'):
            return parent.name
    
    def get_frontend(self):
        """Returns this sardana frontend object or None if no frontend is
        registered
        
        :return: this objects frontend
        :rtype: :obj:`object`"""        
        f = self._frontend
        if f is None:
            return None
        return f()
    
    def fire_event(self, event_type, event_value, listeners=None, protected=True):
        if protected:
            try:
                return EventGenerator.fire_event(self, event_type, event_value,
                                                 listeners=listeners)
            except:
                self.warning("Error firing event <%r, %r>", event_type, event_value)
                self.debug("Details", exc_info=1)
        else:
            return EventGenerator.fire_event(self, event_type, event_value,
                                             listeners=listeners)
    
    def get_interfaces(self):
        """Returns the set of interfaces this object implements.
        
        :return:
            The set of interfaces this object implements.
        :rtype:
            class:`set` <:class:`sardana.sardanadefs.Interface`>"""
        return InterfacesExpanded[self.get_interface()]
    
    def get_interface(self):
        """Returns the interface this object implements.
        
        :return:
            The interface this object implements.
        :rtype:
            :class:`sardana.sardanadefs.Interface`"""
        return Interface[ElementType[self.get_type()]]
    
    def get_interface_names(self):
        """Returns a sequence of interface names this object implements.
        
        :return:
            The sequence of interfaces this object implements.
        :rtype:
            sequence<:obj:`str`>"""
        return map(Interface.get, self.get_interfaces())
    
    def serialize(self, *args, **kwargs):
        kwargs['name'] = self.name
        kwargs['full_name'] = self.full_name
        kwargs['type'] = ElementType.whatis(self.get_type())
        kwargs['manager'] = self.manager.name
        kwargs['parent'] = self.get_parent_name()
        kwargs['interfaces'] = self.get_interface_names()
        return kwargs
    
    def serialized(self, *args, **kwargs):
        return self.manager.serialize_element(self, *args, **kwargs)
    
    def str(self, *args, **kwargs):
        return self.manager.str_element(self, *args, **kwargs)
    
    def __str__(self):
        return self._name
    
    def __repr__(self):
        return "%s(%s)" % (self.__class__.__name__, self._name)
    
    manager = property(get_manager,
                       doc="reference to the :class:`sardana.Manager`")
    name = property(get_name, doc="object name")
    full_name = property(get_full_name, doc="object full name")
    frontend = property(get_frontend, doc="the object frontend")
    

class SardanaObjectID(object):
    """To be used by sardana objects which have an ID associated to them."""
    
    def __init__(self, id=InvalidId):
        self._id = id
    
    def get_id(self):
        """Returns this sardana object ID
        
        :return: this sardana object ID
        :rtype: int"""
        return self._id
    
    def serialize(self, *args, **kwargs):
        kwargs['id'] = self.id
        return kwargs
    
    id = property(get_id, doc="object ID")



##############################################################################
##
## This file is part of Sardana
##
## http://www.tango-controls.org/static/sardana/latest/doc/html/index.html
##
## Copyright 2011 CELLS / ALBA Synchrotron, Bellaterra, Spain
##
## Sardana is free software: you can redistribute it and/or modify
## it under the terms of the GNU Lesser General Public License as published by
## the Free Software Foundation, either version 3 of the License, or
## (at your option) any later version.
##
## Sardana is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU Lesser General Public License for more details.
##
## You should have received a copy of the GNU Lesser General Public License
## along with Sardana.  If not, see <http://www.gnu.org/licenses/>.
##
##############################################################################

"""This file contains the code for an hypothetical Springfield motor hardware
access library. It is intended to be used in the sardana documentation as
an aid to writing a sardana motor controller library.

If you intend to use this code please put it in a directory accessible to
Python or in the same directory as sf_motor_ctrl.py"""

__all__ = ["SpringfieldMotorHW", "SpringfieldCounterHW"]

import time
from math import pow, sqrt

class BaseMotion(object):

    def __init__(self):
        self.min_vel = -1
        self.max_vel = -1
        self.accel_time = -1
        self.decel_time = -1
        self.accel = -1
        self.decel = -1

        self.init_pos = -1
        self.final_pos = -1
        self.curr_pos = -1


class Motion(BaseMotion):

    def __init__(self):
        BaseMotion.__init__(self)

        self.close_loop = False

        self.dsplmnt_reach_max_vel = -1
        self.dsplmnt_reach_min_vel = -1
        self.dsplmnt = -1

        self.curr_instant = -1
        self.start_instant = -1

        self.positive_dsplmnt = True
        self.small_motion = False

        # position where maximum velocity will be reached
        self.curr_max_vel_pos  = -1

        # necessary displacement to reach maximum velocity
        self.curr_dsplmnt_reach_max_vel = -1

        # necessary diplacement to reach minimum velocity
        self.curr_dsplmnt_reach_min_vel = -1

        # maximum velocity possible
        self.curr_max_vel = -1

        # time at maximum velocity
        self.curr_at_max_vel_dsplmnt = -1

        # time to reach maximum velocity
        self.curr_max_vel_time = -1

        # time to reach minimum velocity
        self.curr_min_vel_time = -1

        # time at maximum velocity
        self.curr_at_max_vel_time = -1

        # instant when maximum velocity should be reached
        self.curr_max_vel_instant = -1

        # instant when should start decelerating
        self.curr_min_vel_instant = -1

        # time the motion will take
        self.duration = -1

        # instant the motion will end
        self.final_instant = -1

        # steps per unit
        self.step_per_unit = 1

        self.inMotion = False

        self.lower_ls = float('-inf')
        self.upper_ls = float('+inf')

        self.power = True
        self.enabled = True

        self.__recalculate_acc_constants()

    def isCloseLoopActive(self):
        return self.close_loop
        
    def setCloseLoop(self, v):
        self.close_loop = v

    def setMinVelocity(self,vi):
        """ Sets the minimum velocity in ms^-1. A.k.a. base rate"""
        vi = float(vi)
        if vi < 0:
            raise "Minimum velocity must be >= 0"

        self.min_vel = vi

        if self.max_vel < self.min_vel:
            self.max_vel = self.min_vel

        # force recalculation of accelerations
        if self.accel_time >= 0:
            self.setAccelerationTime(self.accel_time)
        if self.decel_time >= 0:
            self.setDecelerationTime(self.decel_time)

    def getMinVelocity(self):
        return self.min_vel

    def setMaxVelocity(self,vf):
        """ Sets the maximum velocity in ms^-1."""
        vf = float(vf)
        if vf <= 0:
            raise "Maximum velocity must be > 0"

        self.max_vel = vf

        if self.min_vel > self.max_vel:
            self.min_vel = self.max_vel

        # force recalculation of accelerations
        if self.accel_time >= 0:
            self.setAccelerationTime(self.accel_time)
        if self.decel_time >= 0:
            self.setDecelerationTime(self.decel_time)

    def getMaxVelocity(self):
        return self.max_vel

    def setAccelerationTime(self,at):
        """Sets the time to go from minimum velocity to maximum velocity in seconds"""
        at = float(at)
        if at <= 0:
            raise "Acceleration time must be > 0"

        self.accel_time = at
        self.accel = (self.max_vel - self.min_vel) / at

        self.__recalculate_acc_constants()

    def getAccelerationTime(self):
        return self.accel_time

    def setDecelerationTime(self,dt):
        """Sets the time to go from maximum velocity to minimum velocity in seconds"""
        dt = float(dt)
        if dt <= 0:
            raise "Deceleration time must be > 0"

        self.decel_time = dt
        self.decel = (self.min_vel - self.max_vel) / dt

        self.__recalculate_acc_constants()

    def getDecelerationTime(self):
        return self.decel_time

    def setAcceleration(self,a):
        """Sets the acceleration in ms^-2"""
        a = float(a)
        if a < 0:
            raise "Acceleration must be >= 0"

        self.accel = float(a)

        if a > 0:
            self.accel_time = (self.max_vel - self.min_vel) / a
        else:
            self.accel_time = float('INF')

        self.__recalculate_acc_constants()

    def setDeceleration(self,d):
        """Sets the deceleration in ms^-2"""
        d = float(d)
        if d > 0:
            raise "Deceleration must be <= 0"

        self.decel = d

        if d < 0:
            self.decel_time = (self.min_vel - self.max_vel) / d
        else:
            self.decel_time = float('INF')

        self.__recalculate_acc_constants()

    def getStepPerUnit(self):
        return self.step_per_unit

    def setStepPerUnit(self, spu):
        self.step_per_unit = spu

    def __recalculate_acc_constants(self):
        """precomputations assuming maximum speed can be reached in a motion"""

        self.dsplmnt_reach_max_vel = 0.5 * self.accel * pow(self.accel_time,2)
        self.dsplmnt_reach_max_vel += self.min_vel * self.accel_time

        self.dsplmnt_reach_min_vel = 0.5 * self.decel * pow(self.decel_time,2)
        self.dsplmnt_reach_min_vel += self.max_vel * self.decel_time

    def startMotion(self, initial_user_pos, final_user_pos, start_instant=None):
        """starts a new motion"""

        if not self.power:
            raise Exception("Motor is powered off")

        initial_pos = initial_user_pos * self.step_per_unit
        final_pos = final_user_pos * self.step_per_unit

        if self.inMotion:
            raise Exception("Already in motion")

        if initial_pos == final_pos:
            return

        self.init_pos = initial_pos
        self.final_pos = final_pos
        self.curr_pos = initial_pos
        self.dsplmnt = abs(final_pos - initial_pos)

        start_instant = start_instant or time.time()
        self.curr_instant = start_instant
        self.start_instant = start_instant

        self.positive_dsplmnt = final_pos >= initial_pos

        displmnt_not_cnst = self.dsplmnt_reach_max_vel + self.dsplmnt_reach_min_vel
        self.small_motion = self.dsplmnt < displmnt_not_cnst

        if self.positive_dsplmnt:
            self.curr_accel = self.accel
            self.curr_decel = self.decel
        else:
            self.curr_accel = -self.accel
            self.curr_decel = -self.decel


        if not self.small_motion:

            # necessary displacement to reach maximum velocity
            self.curr_dsplmnt_reach_max_vel = self.dsplmnt_reach_max_vel
            # necessary diplacement to reach minimum velocity
            self.curr_dsplmnt_reach_min_vel = self.dsplmnt_reach_min_vel

            if self.positive_dsplmnt:
                self.curr_max_vel = self.max_vel
                self.curr_min_vel = self.min_vel
                # position where maximum velocity will be reached
                self.curr_max_vel_pos = self.init_pos + self.curr_dsplmnt_reach_max_vel
            else:
                self.curr_max_vel = -self.max_vel
                self.curr_min_vel = -self.min_vel
                # position where maximum velocity will be reached
                self.curr_max_vel_pos = self.init_pos - self.curr_dsplmnt_reach_max_vel

            # displacement at maximum velocity
            self.curr_at_max_vel_dsplmnt = self.dsplmnt - (self.curr_dsplmnt_reach_max_vel + self.curr_dsplmnt_reach_min_vel)

        else:  # Small movement
            # position where maximum velocity will be reached
            self.curr_max_vel_pos  = self.init_pos * self.curr_accel - self.final_pos * self.curr_decel
            self.curr_max_vel_pos /= self.curr_accel - self.curr_decel

            # necessary displacement to reach maximum velocity
            self.curr_dsplmnt_reach_max_vel = abs(self.curr_max_vel_pos - self.init_pos)

            # necessary diplacement to reach minimum velocity
            self.curr_dsplmnt_reach_min_vel = abs(self.final_pos - self.curr_max_vel_pos)

            # maximum velocity possible
            cnst = 2 * self.curr_accel * self.curr_decel * self.dsplmnt / (self.curr_decel - self.curr_accel)
            max_vel_2 = pow(self.min_vel, 2) + cnst

            self.curr_max_vel = sqrt(abs(max_vel_2))

            if self.positive_dsplmnt:
                self.curr_min_vel = self.min_vel
            else:
                self.curr_max_vel = -self.curr_max_vel
                self.curr_min_vel = -self.min_vel

            # displacement at maximum velocity
            self.curr_at_max_vel_dsplmnt = 0.0

        # time to reach maximum velocity
        self.curr_max_vel_time = abs((self.curr_max_vel - self.curr_min_vel) / self.curr_accel)

        # time to reach minimum velocity
        self.curr_min_vel_time = abs((self.curr_min_vel - self.curr_max_vel) / self.curr_decel)

        # time at maximum velocity
        self.curr_at_max_vel_time = abs(self.curr_at_max_vel_dsplmnt / self.curr_max_vel)

        # instant when maximum velocity should be reached
        self.curr_max_vel_instant = self.start_instant + self.curr_max_vel_time

        # instant when should start decelerating
        self.curr_min_vel_instant = self.curr_max_vel_instant + self.curr_at_max_vel_time

        # time the motion will take
        self.duration = self.curr_max_vel_time + self.curr_at_max_vel_time + self.curr_min_vel_time

        # instant the motion will end
        self.final_instant = self.start_instant + self.duration

        # uncomment following line if need output concerning the movement that
        # has just started
        # self.info()

        # ASSERTIONS
        if self.positive_dsplmnt:
            assert(self.curr_max_vel_pos >= self.init_pos)
            assert(self.curr_max_vel_pos <= self.final_pos)
        else:
            assert(self.curr_max_vel_pos <= self.init_pos)
            assert(self.curr_max_vel_pos >= self.final_pos)

        assert(self.curr_dsplmnt_reach_max_vel >= 0.0)
        assert(self.curr_dsplmnt_reach_min_vel >= 0.0)

        assert(self.final_instant >= self.start_instant)
        assert(self.curr_max_vel <= self.max_vel)
        assert(self.start_instant <= self.curr_max_vel_instant)
        assert(self.final_instant >= self.curr_min_vel_instant)

        assert(self.curr_max_vel_time > 0.0)
        assert(self.curr_min_vel_time > 0.0)
        assert(self.duration > 0.0)

        if self.small_motion:
            assert(self.curr_max_vel_instant == self.curr_min_vel_instant)
            assert(self.curr_at_max_vel_time == 0.0)
        else:
            assert(self.curr_max_vel_instant <= self.curr_min_vel_instant)
            assert(self.curr_at_max_vel_time >= 0.0)

        self.inMotion = True

    def abortMotion(self, curr_instant=None):
        curr_instant = curr_instant or time.time()
        if not self.inMotion:
            return self.curr_pos

        self.curr_pos = self.getCurrentPosition(curr_instant)
        self.inMotion = False
        return self.curr_pos

    def isInMotion(self,curr_instant=None):
        curr_instant = curr_instant or time.time()
        #we call getCurrentPosition because inside it updates the inMotion flag
        self.getCurrentPosition(curr_instant)
        return self.inMotion

    def setCurrentPosition(self, curr_pos):
        self.curr_pos = curr_pos
        self.init_pos = curr_pos

    def getCurrentPosition(self, curr_instant=None):
        curr_instant = curr_instant or time.time()
        self.curr_instant = curr_instant
        pos = None
        if self.inMotion:
            # if motion should be ended...
            if self.curr_instant >= self.final_instant:
                self.inMotion = False
                pos = self.final_pos
            else:
                pos  = self.init_pos
                if curr_instant > self.curr_min_vel_instant:
                    if self.positive_dsplmnt:
                        pos += self.curr_dsplmnt_reach_max_vel
                        pos += self.curr_at_max_vel_dsplmnt
                    else:
                        pos -= self.curr_dsplmnt_reach_max_vel
                        pos -= self.curr_at_max_vel_dsplmnt
                    dt = curr_instant - self.curr_min_vel_instant
                    pos += self.curr_max_vel * dt + 0.5 * self.curr_decel * pow(dt,2)
                elif curr_instant > self.curr_max_vel_instant:
                    if self.positive_dsplmnt:
                        pos += self.curr_dsplmnt_reach_max_vel
                    else:
                        pos -= self.curr_dsplmnt_reach_max_vel
                    dt = curr_instant - self.curr_max_vel_instant
                    pos += self.curr_max_vel * dt
                else:
                    dt  = curr_instant - self.start_instant
                    pos += self.curr_min_vel * dt + 0.5 * self.curr_accel * pow(dt,2)
        else:
            pos = self.curr_pos
        if pos <= self.lower_ls:
            pos = self.lower_ls
            self.inMotion = False
        elif pos >= self.upper_ls:
            pos = self.upper_ls
            self.inMotion = False
        self.curr_pos = pos
        return pos

    def setCurrentUserPosition(self, user_pos):
        self.setCurrentPosition(user_pos*self.step_per_unit)

    def getCurrentUserPosition(self, curr_instant=None):
        return self.getCurrentPosition(curr_instant=curr_instant) / self.step_per_unit

    def hitLowerLimit(self):
        user_pos = self.curr_pos / self.step_per_unit
        return user_pos <= self.lower_ls

    def hitUpperLimit(self):
        user_pos = self.curr_pos / self.step_per_unit
        return user_pos >= self.upper_ls

    def getLowerLimitSwitch(self):
        return self.lower_ls

    def setLowerLimitSwitch(self, user_lower_ls):
        self.lower_ls = user_lower_ls

    def getUpperLimitSwitch(self):
        return self.upper_ls

    def setUpperLimitSwitch(self, user_upper_ls):
        self.upper_ls = user_upper_ls

    def turnOn(self):
        self.power = True

    def turnOff(self):
        self.power = False

    def isTurnedOn(self):
        return self.power

    def hasPower(self):
        return self.power

    def setPower(self, power):
        self.power = power

    def info(self):
        print "Small movement =",self.small_motion
        print "length =",self.dsplmnt
        print "position where maximum velocity will be reached =",self.curr_max_vel_pos
        print "necessary displacement to reach maximum velocity =",self.curr_dsplmnt_reach_max_vel
        print "necessary displacement to stop from maximum velocity =",self.curr_dsplmnt_reach_min_vel
        print "maximum velocity possible =",self.curr_max_vel
        print "time at top velocity =",self.curr_at_max_vel_time
        print "displacement at top velocity =",self.curr_at_max_vel_dsplmnt
        print "time to reach maximum velocity =",self.curr_max_vel_time
        print "time to reach minimum velocity =",self.curr_min_vel_time
        print "time the motion will take =",self.duration
        print "instant when maximum velocity should be reached =",self.curr_max_vel_instant
        print "instant when should start decelerating =",self.curr_min_vel_instant
        print "instant the motion will end",self.final_instant
        print ""
        print "For long movements (where top vel is possible), necessary displacement to reach maximum velocity =",self.dsplmnt_reach_max_vel
        print "For long movements (where top vel is possible), necessary displacement to stop from maximum velocity =",self.dsplmnt_reach_min_vel


class SpringfieldMotorHW(object):
    
    DefaultHost = "localhost"
    DefaultPort = 10123
    
    def __init__(self, host=DefaultHost, port=DefaultPort):
        self.host = host
        self.port = port
        self._motions = {}
        
    def getMotion(self, axis):
        motion = self._motions.get(axis)
        if motion is None:
            self._motions[axis] = motion = Motion()
        return motion
        
    def getState(self, axis):
        motion = self.getMotion(axis)
        motion.getCurrentUserPosition()
        if motion.isInMotion():
            return 2
        if motion.hitLowerLimit():
            return 3
        if motion.hitUpperLimit():
            return 3
        if not motion.hasPower():
            return 4
        return 1
    
    def getStatus(self, axis):
        motion = self.getMotion(axis)
        motion.getCurrentUserPosition()
        status = "Motor HW is ON"
        if motion.isInMotion():
            status = "Motor HW is MOVING"
        if motion.hitLowerLimit():
            status = "Motor HW is in ALARM. Hit hardware lower limit switch"
        if motion.hitUpperLimit():
            status = "Motor HW is in ALARM. Hit hardware upper limit switch"
        if not motion.hasPower():
            status = "Motor is powered off"
        return status

    def getLimits(self, axis):
        motion = self.getMotion(axis)
        m.getCurrentUserPosition()
        switchstate = 3*[False,]
        if m.hitLowerLimit():
            switchstate[2] = True
        if m.hitUpperLimit():
            switchstate[1] = True
        return switchstate
        
    def getPosition(self, axis):
        motion = self.getMotion(axis)
        return motion.getCurrentUserPosition()

    def getAccelerationTime(self, axis):
        return self.getMotion(axis).getAccelerationTime()

    def getDecelerationTime(self, axis):
        return self.getMotion(axis).getDecelerationTime()

    def getMinVelocity(self, axis):
        return self.getMotion(axis).getMinVelocity()

    def getMaxVelocity(self, axis):
        return self.getMotion(axis).getMaxVelocity()

    def getStepPerUnit(self, axis):
        return self.getMotion(axis).getStepPerUnit()
        
    def setAccelerationTime(self, axis, v):
        self.getMotion(axis).setAccelerationTime(v)

    def setDecelerationTime(self, axis, v):
        self.getMotion(axis).setDecelerationTime(v)

    def setMinVelocity(self, axis, v):
        self.getMotion(axis).setMinVelocity(v)

    def setMaxVelocity(self, axis, v):
        self.getMotion(axis).setMaxVelocity(v)

    def setStepPerUnit(self, axis, v):
        self.getMotion(axis).setStepPerUnit(v)

    def isCloseLoopActive(self, axis):
        return self.getMotion(axis).isCloseLoopActive()

    def setCloseLoop(self, axis, v):
        self.getMotion(axis).setCloseLoop(v)

    def setCurrentPosition(self, axis, position):
        motion = self.getMotion(axis)
        motion.offset = position - motion.getCurrentUserPosition()
        motion.setCurrentUserPosition(position)

    def move(self, axis, position):
        t = time.time()
        motion = self.getMotion(axis)
        motion.startMotion(motion.getCurrentUserPosition(t), position, t)
    
    def stop(self, axis):
        motion = self.getMotion(axis)
        motion.abortMotion()

    def abort(self, axis):
        motion = self.getMotion(axis)
        motion.abortMotion()
        

class Channel:
    
    def __init__(self,idx):
        self.idx = idx            # 1 based index
        self.value = 0.0
        self.is_counting = False
        self.active = True

    
class SpringfieldCounterHW(object):

    DefaultHost = "localhost"
    DefaultPort = 10124
    
    def __init__(self, host=DefaultHost, port=DefaultPort):
        self.host = host
        self.port = port
        self._channels = {}
        
    def getChannel(self, axis):
        channel = self._channels.get(axis)
        if channel is None:
            self._channels[axis] = channel = Channel(axis)
        return channel
        
    def getState(self, axis):
        channel = self.getChannel(axis)
        channel.getCurrentUserValue()
        if channel.isAcquiring():
            return 2
        if not channel.hasPower():
            return 3
        return 1
    
    def getStatus(self, axis):
        channel = self.getChannel(axis)
        channel.getCurrentUserValue()
        status = "Counter HW is ON"
        if channel.isAcquiring():
            status = "Counter HW is ACQUIRING"
        if not channel.hasPower():
            status = "Counter is powered OFF"
        return status

    def getValue(self, axis):
        motion = self.getMotion(axis)
        return motion.getCurrentUserPosition()    


#!/usr/bin/env python

##############################################################################
##
## This file is part of Sardana
##
## http://www.tango-controls.org/static/sardana/latest/doc/html/index.html
##
## Copyright 2011 CELLS / ALBA Synchrotron, Bellaterra, Spain
##
## Sardana is free software: you can redistribute it and/or modify
## it under the terms of the GNU Lesser General Public License as published by
## the Free Software Foundation, either version 3 of the License, or
## (at your option) any later version.
##
## Sardana is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU Lesser General Public License for more details.
##
## You should have received a copy of the GNU Lesser General Public License
## along with Sardana.  If not, see <http://www.gnu.org/licenses/>.
##
##############################################################################

"""This module contains the class definition for the macro server door"""

__all__ = ["MacroProxy", "BaseInputHandler", "MSDoor"]

__docformat__ = 'restructuredtext'

import collections
import weakref

from taurus.core.util import Logger

from sardana import ElementType
from sardana.sardanaevent import EventType

from msbase import MSObject
from msparameter import Type


class MacroProxy(object):

    def __init__(self, door, macro_meta):
        self._door = weakref.ref(door)
        self._macro_meta = weakref.ref(macro_meta)

    @property
    def door(self):
        return self._door()

    @property
    def macro_info(self):
        return self._macro_meta()

    def __call__(self, *args, **kwargs):
        door = self.door
        parent_macro = door.get_running_macro()
        parent_macro.syncLog()
        executor = parent_macro.executor
        opts=dict(parent_macro=parent_macro, executor=executor)
        kwargs.update(opts)
        eargs = [self.macro_info.name]
        eargs.extend(args)
        return parent_macro.execMacro(*eargs, **kwargs)


class MacroProxyCache(dict):

    def __init__(self, door):
        self._door = weakref.ref(door)
        self.rebuild()

    @property
    def door(self):
        return self._door()

    def rebuild(self):
        self.clear()
        door = self.door
        macros = self.door.get_macros()
        for macro_name, macro_meta in macros.items():
            self[macro_name] = MacroProxy(door, macro_meta)


class BaseInputHandler(object):

    def __init__(self):
        try:
            self._input = raw_input
        except NameError:
            self._input = input

    def input(self, input_data=None):
        if input_data is None:
            input_data = {}
        prompt = input_data.get('prompt')
        if prompt is None:
            return self._input()
        else:
            return self._input(prompt)


class MSDoor(MSObject):
    """Sardana door object"""

    def __init__(self, **kwargs):
        self._state = None
        self._status = None
        self._result = None
        self._macro_status = None
        self._record_data = None
        self._macro_proxy_cache = None
        self._input_handler = BaseInputHandler()
        self._pylab_handler = None
        kwargs['elem_type'] = ElementType.Door
        MSObject.__init__(self, **kwargs)

    def get_macro_executor(self):
        return self.macro_server.macro_manager.getMacroExecutor(self)

    macro_executor = property(get_macro_executor)

    def get_running_macro(self):
        return self.macro_executor.getRunningMacro()

    running_macro = property(get_running_macro)

    def get_macro_data(self):
        macro = self.running_macro
        if macro is None:
            raise Exception("No macro has run so far!")
        data = macro.data
        return data

    def set_pylab_handler(self, ph):
        self._pylab_handler = ph
    
    def get_pylab_handler(self):
        return self._pylab_handler

    pylab_handler = property(get_pylab_handler, set_pylab_handler)
    
    def get_pylab(self):
        ph = self.pylab_handler
        if ph is None:
            import matplotlib.pylab
            ph = matplotlib.pylab
        return ph

    pylab = property(get_pylab)

    def set_pyplot_handler(self, ph):
        self._pyplot_handler = ph
    
    def get_pyplot_handler(self):
        return self._pyplot_handler

    pyplot_handler = property(get_pyplot_handler, set_pyplot_handler)
    
    def get_pyplot(self):
        ph = self.pyplot_handler
        if ph is None:
            import matplotlib.pyplot
            ph = matplotlib.pyplot
        return ph

    pyplot = property(get_pyplot)

    def set_input_handler(self, ih):
        self._input_handler = ih

    def get_input_handler(self):
        return self._input_handler

    input_handler = property(get_input_handler, set_input_handler)

    def append_prompt(self, prompt, msg):
        if '?' in prompt:
            prefix, suffix = prompt.rsplit('?', 1)
            if not prefix.endswith(' '):
                prefix += ' '
            prompt = prefix + msg + '?' + suffix
        else:
            prompt += msg + ' '
        return prompt

    def input(self, msg, *args, **kwargs):
        kwargs['data_type'] = kwargs.get('data_type', Type.String)
        kwargs['allow_multiple'] = kwargs.get('allow_multiple', False)
        
        if args:
            msg = msg % args
        if not msg.endswith(' '):
            msg += ' '
        dv = kwargs.get('default_value')
        if dv is not None:
            dv = '[' + str(dv) + ']'
            msg = self.append_prompt(msg, dv)

        macro = kwargs.pop('macro', self.macro_executor.getRunningMacro())
        if macro is None:
            macro = self

        input_data = dict(prompt=msg, type='input')
        input_data.update(kwargs)
        data_type = kwargs['data_type']
        is_seq = not isinstance(data_type, (str, unicode)) and \
                 isinstance(data_type, collections.Sequence)
        if is_seq:
            handle = self._handle_seq_input
        else:
            handle = self._handle_type_input
        
        return handle(macro, input_data, data_type)

    def _handle_seq_input(self, obj, input_data, data_type):
        valid = False
        allow_multiple = input_data['allow_multiple']
        while not valid:
            result = self.input_handler.input(input_data)
            if allow_multiple:
                r, dt = set(result), set(data_type)
                if r.issubset(dt):
                    break
            else:
                if result in data_type:
                    break
            obj.warning("Please give a valid option")
        return result
    
    def _handle_type_input(self, obj, input_data, data_type):
        type_obj = self.type_manager.getTypeObj(data_type)

        valid = False
        while not valid:
            result = self.input_handler.input(input_data)
            try:
                result_type = type_obj.getObj(result)
                if result_type is None:
                    raise Exception("Must give a value")
                valid = True
                return result_type
            except:
                dtype = str(data_type).lower()
                obj.warning("Please give a valid %s.", dtype)

    def get_report_logger(self):
        return self.macro_server.report_logger

    report_logger = property(get_report_logger)

    def report(self, msg, *args, **kwargs):
        """
        Record a log message in the sardana report (if enabled) with default
        level **INFO**. The msg is the message format string, and the args are
        the arguments which are merged into msg using the string formatting
        operator. (Note that this means that you can use keywords in the
        format string, together with a single dictionary argument.)

        *kwargs* are the same as :meth:`logging.Logger.debug` plus an optional
        level kwargs which has default value **INFO**

        Example::

            self.report("this is an official report!")

        :param msg: the message to be recorded
        :type msg: :obj:`str`
        :param args: list of arguments
        :param kwargs: list of keyword arguments"""
        return self.macro_server.report(msg, *args, **kwargs)

    def get_state(self):
        return self._state

    def set_state(self, state, propagate=1):
        self._state = state
        self.fire_event(EventType("state", priority=propagate), state)

    state = property(get_state, set_state)

    def get_status(self):
        return self._status

    def set_status(self, status, propagate=1):
        self._status = status
        self.fire_event(EventType("status", priority=propagate), status)

    status = property(get_status, set_status)

    def get_result(self):
        return self._result

    def set_result(self, result, propagate=1):
        self._result = result
        self.fire_event(EventType("result", priority=propagate), result)

    result = property(get_result, set_result)

    def get_macro_status(self):
        return self._macro_status

    def set_macro_status(self, macro_status, propagate=1):
        self._macro_status = macro_status
        self.fire_event(EventType("macrostatus", priority=propagate),
                        macro_status)

    result = property(get_result, set_result)

    def get_record_data(self):
        return self._record_data

    def set_record_data(self, record_data, codec=None, propagate=1):
        self._record_data = record_data
        self.fire_event(EventType("recorddata", priority=propagate),
                        (codec, record_data))

    record_data = property(get_record_data, set_record_data)

    def get_env(self, key=None, macro_name=None):
        """Gets the environment with the context for this door matching the
        given parameters:

        - macro_name defines the context where to look for the environment. If
          None, the global environment is used. If macro name is given the
          environment in the context of that macro is given
        - If key is None it returns the complete environment, otherwise
          key must be a string containing the environment variable name.

        :param key:
            environment variable name [default: None, meaning all environment]
        :type key: str
        :param macro_name:
            local context for a given macro [default: None, meaning no macro
            context is used]
        :type macro_name: str

        :raises: UnknownEnv"""
        return self.macro_server.environment_manager.getAllDoorEnv(self.name)

    def set_env(self, key, value):
        return self.macro_server.set_env(key, value)

    def _build_macro_proxy_cache(self):
        self._macro_proxy_cache = MacroProxyCache(self)

    def get_macro_proxies(self):
        if self._macro_proxy_cache is None:
            self._macro_proxy_cache = MacroProxyCache(self)
        return self._macro_proxy_cache

    def run_macro(self, par_str_list, asynch=False):
        if isinstance(par_str_list, (str, unicode)):
            par_str_list = par_str_list,

        if not hasattr(self, "Output"):
            import sys
            import logging
            Logger.addLevelName(15, "OUTPUT")

            def output(loggable, msg, *args, **kw):
                loggable.getLogObj().log(Logger.Output, msg, *args, **kw)
            Logger.output = output

            Logger.disableLogOutput()
            Logger.setLogLevel(Logger.Output)
            #filter = taurus.core.util.LogFilter(level=Logger.Output)
            formatter = logging.Formatter(fmt="%(message)s")
            Logger.setLogFormat("%(message)s")
            handler = logging.StreamHandler(stream=sys.stdout)
            #handler.addFilter(filter)
            Logger.addRootLogHandler(handler)
            #handler.setFormatter(formatter)
            #logger.addHandler(handler)
            #logger.addFilter(filter)
            self.__logging_info = handler, filter, formatter

            # result of a macro
            #Logger.addLevelName(18, "RESULT")

        return self.macro_executor.run(par_str_list, asynch=asynch)

    def __getattr__(self, name):
        """Get methods from macro server"""
        return getattr(self.macro_server, name)



#!/usr/bin/env python

##############################################################################
##
## This file is part of Sardana
##
## http://www.tango-controls.org/static/sardana/latest/doc/html/index.html
##
## Copyright 2011 CELLS / ALBA Synchrotron, Bellaterra, Spain
## 
## Sardana is free software: you can redistribute it and/or modify
## it under the terms of the GNU Lesser General Public License as published by
## the Free Software Foundation, either version 3 of the License, or
## (at your option) any later version.
## 
## Sardana is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU Lesser General Public License for more details.
## 
## You should have received a copy of the GNU Lesser General Public License
## along with Sardana.  If not, see <http://www.gnu.org/licenses/>.
##
##############################################################################

"""This module contains the definition of the macroserver data type manager"""

__all__ = ["TypeManager"]

__docformat__ = 'restructuredtext'

import os
import inspect

from sardana.sardanamodulemanager import ModuleManager

from msparameter import Type, ParamType, AbstractParamTypes
from msmanager import MacroServerManager

_BASE_DIR = os.path.dirname(os.path.abspath(__file__))


class TypeManager(MacroServerManager):
    
    DEFAULT_TYPE_DIR = _BASE_DIR
    DEFAULT_TYPE_MODULES = 'basetypes',
    
    def reInit(self):
        if self.is_initialized():
            return

        # dict<str, dict<str,class<ParamType>>
        # key   - module name (without path and without extension)
        # value - a dict where:
        #         key   - type name
        #         value - class object implementing the Type
        self._modules = {}
        
        # dict<str, ParamType>
        # key   - type name
        # value - object which inherits from ParamType
        self._inst_dict = {}
        
        path = [ self.DEFAULT_TYPE_DIR ]
        for type_module in self.DEFAULT_TYPE_MODULES:
            self.reloadTypeModule(type_module, path=path)
        
        MacroServerManager.reInit(self)
    
    def cleanUp(self):
        if self.is_cleaned():
            return
        
        if self._modules:
            for _, types_dict in self._modules.items():
                for type_name in types_dict:
                    Type.removeType(type_name)
                
        self._modules = None
        self._inst_dict = None
        
        MacroServerManager.cleanUp(self)
        
    def getTypeListObj(self):
        return self._type_list_obj
    
    def reloadTypeModule(self, module_name, path=None):
        """Loads/reloads the given module name"""
        #path = path or [ os.path.dirname(__file__) ]
        m = None
        try:
            m = ModuleManager().reloadModule(module_name, path)
        except:
            pass
        
        if m is None:
            if self._modules.has_key(module_name):
                self._modules.pop(module_name)
            return
        
        self._modules[module_name] = {}
        
        abs_file = inspect.getabsfile(m)
        ms = self.macro_server
        for name in dir(m):
            if name.startswith("_"):
                continue
            klass = getattr(m, name)
            try:
                if not issubclass(klass, ParamType):
                    continue
            except:
                continue
            if klass in AbstractParamTypes:
                continue
            if inspect.getabsfile(klass) != abs_file:
                continue
            
            t = klass(ms, name)
            self.addType(t)
    
    def addType(self, type_obj):
        type_name = type_obj.getName()
        type_class = type_obj.__class__
        module_name = type_obj.__module__
        
        mod_types = self._modules[module_name]
        
        #action = (((type_name in mod_types) and "Updating") \
        #          or "Adding")
        action = "Updating"
        self.debug("%s type %s", action, type_name)
        mod_types[type_name] = type_class
        self._inst_dict[type_name] = type_obj
        
        Type.addType(type_name)
    
    def getTypeListStr(self):
        type_list_basic, type_list_obj = [], []
        
        for module_name, type_class_dict in self._modules.items():
            for tname, tklass in type_class_dict.items():
                if tklass.hasCapability(ParamType.ItemList):
                    type_list_obj.append("%s*" % tname)
                else:
                    type_list_basic.append(tname)
        type_list = sorted(type_list_basic) + sorted(type_list_obj)
        
        return type_list
    
    def getTypeClass(self, type_name):
        for module_name, type_class_dict in self._modules.items():
            tklass = type_class_dict.get(type_name)
            if tklass is None:
                continue
            return tklass
        return None
    
    def getTypeObj(self, type_name):
        return self._inst_dict.get(type_name)
    
    def getTypes(self):
        return self._inst_dict
    
    def getTypeNames(self):
        return self._inst_dict.keys()


import math
from numpy import linspace
from scipy.integrate import quad
from scipy.special import j0

from sardana.macroserver.macro import macro, Type

def j0i(x):
    """Integral form of J_0(x)"""
    def integrand(phi):
        return math.cos(x * math.sin(phi))
    return (1.0/math.pi) * quad(integrand, 0, math.pi)[0]

@macro()
def J0_plot(self):
    """Sample J0 at linspace(0, 20, 200)"""
    x = linspace(0, 20, 200)
    y = j0(x)
    x1 = x[::10]
    y1 = map(j0i, x1)
    self.pyplot.plot(x, y, label=r'$J_0(x)$') # 
    self.pyplot.plot(x1, y1, 'ro', label=r'$J_0^{integ}(x)$')
    self.pyplot.title(r'Verify $J_0(x)=\frac{1}{\pi}\int_0^{\pi}\cos(x \sin\phi)\,d\phi$')
    self.pyplot.xlabel('$x$')
    self.pyplot.legend()


from numpy import random

@macro()
def random_image(self):
    """Shows a random image 32x32"""
    img = random.random((32, 32))
    self.pyplot.matshow(img)

import numpy
    
@macro([["interactions", Type.Integer, None, ""],
        ["density", Type.Integer, None, ""]])
def mandelbrot(self, interactions, density):

    x_min, x_max = -2, 1
    y_min, y_max = -1.5, 1.5
    
    x, y = numpy.meshgrid(numpy.linspace(x_min, x_max, density),
                          numpy.linspace(y_min, y_max, density))
                         
    c = x + 1j * y
    z = c.copy()
    
    fractal = numpy.zeros(z.shape, dtype=numpy.uint8) + 255
    
    finteractions = float(interactions)
    for n in range(interactions):
        z *= z
        z += c
        mask = (fractal == 255) & (abs(z) > 10)
        fractal[mask] = 254 * n / finteractions
    self.pyplot.imshow(fractal)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

##############################################################################
##
## This file is part of Sardana
## 
## http://www.tango-controls.org/static/sardana/latest/doc/html/index.html
##
## Copyright 2011 CELLS / ALBA Synchrotron, Bellaterra, Spain
## 
## Sardana is free software: you can redistribute it and/or modify
## it under the terms of the GNU Lesser General Public License as published by
## the Free Software Foundation, either version 3 of the License, or
## (at your option) any later version.
## 
## Sardana is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU Lesser General Public License for more details.
## 
## You should have received a copy of the GNU Lesser General Public License
## along with Sardana.  If not, see <http://www.gnu.org/licenses/>.
##
##############################################################################

"""Release data for the Spock project.
"""

# Name of the package for release purposes.  This is the name which labels
# the tarballs and RPMs made by distutils, so it's best to lowercase it.
name = 'spock'

# For versions with substrings (like 0.6.16.svn), use an extra . to separate
# the new substring.  We have to avoid using either dashes or underscores,
# because bdist_rpm does not accept dashes (an RPM) convention, and
# bdist_deb does not accept underscores (a Debian convention).

revision = '1'

#version = '0.8.1.svn.r' + revision.rstrip('M')
version = '1.0.0'

description = "An enhanced interactive Macro Server shell."

long_description = \
"""
Spock provides an interactive environment for interacting with the Tango
MacroServer Device. It is completely based on IPython which itself provides a 
replacement for the interactive Python interpreter with extra functionality.
 """

license = 'GNU'

authors = {'Tiago'          : ('Tiago Coutinho','tiago.coutinho@esrf.fr'),
           'Reszela'        : ('Zbigniew Reszela','zreszela@cells.es') ,
           'Pascual-Izarra' : ('Carlos Pascual-Izarra','cpascual@cells.es') }

url = ''

download_url = ''

platforms = ['Linux','Windows XP/2000/NT','Windows 95/98/ME']

keywords = ['Sardana', 'Interactive', 'MacroServer', 'Tango', 'Shell']



import os
import xml.dom.minidom

from sardana.macroserver.macro import *

TAG_MACRO       = 'macro'
TAG_PARAM       = 'param' 
TAG_PARAMREPEAT = 'paramrepeat'
TAG_REPEAT      = 'repeat'
TAG_PARAMS      = 'params'
TAG_SEQUENCE    = 'sequence'

ATTRIBUTE_NAME          = 'name'
ATTRIBUTE_VALUE         = 'value'
ATTRIBUTE_INDEX         = 'nr'
ATTRIBUTE_DESCRIPTION   = 'description'
ATTRIBUTE_DEFVALUE      = 'defvalue'
ATTRIBUTE_TYPE          = 'type'
ATTRIBUTE_ALLOWEDHOOKS  = 'allowedHooks'
ATTRIBUTE_HASPARAMS     = 'hasParams'
ATTRIBUTE_MIN           = 'min'

class sequence(Macro):
    """This macro executes a sequence of macros. As a parameter 
    it receives a string which is a xml structure. These macros which allow
    hooks can nest another sequence (xml structure). In such a case, 
    this macro is executed recursively."""
    
    param_def = [
       ['xml',   Type.String,   None, 'Xml string representing a sequence']
    ]
        
    def run(self, *pars):       
        xmlDoc = xml.dom.minidom.parseString(pars[0])
        macros = self.parseXml(xmlDoc)
        for macro in macros:
            self.runMacro(macro)
#            self.pausePoint()
            
    def parseXml(self, xmlDoc):
        macros = []
        sequenceElement = xmlDoc.getElementsByTagName(TAG_SEQUENCE)[0]
        childElement = sequenceElement.firstChild
        while childElement:
            if childElement.localName == TAG_MACRO:
                params, hookElement = self.parseMacro(childElement)
                macro = self.createMacro(params)
                if hookElement is not None:
                    hook = self.createExecMacroHook([self.__class__.__name__,
                                                    hookElement.toxml()])
                    macro.hooks = [hook]
                macros.append(macro)
            childElement = childElement.nextSibling
        return macros
            
    def parseMacro(self, xmlElement):
        name = str(xmlElement.getAttribute(ATTRIBUTE_NAME))
        params = (name,)
        hookElement = None
        childElement = xmlElement.firstChild
        while childElement:
            if childElement.localName == TAG_PARAM:
                params += self.parseParam(childElement)
            elif childElement.localName == TAG_PARAMREPEAT:
                params += self.parseParamRepeat(childElement)
            elif childElement.localName == TAG_SEQUENCE:
                hookElement = childElement
            childElement = childElement.nextSibling
        return (params,hookElement)
            
    def parseParam(self, xmlElement):
        return (str(xmlElement.getAttribute(ATTRIBUTE_VALUE)),)
        
    def parseParamRepeat(self, xmlElement):
        params = ()
        childElement = xmlElement.firstChild
        while childElement:
            if childElement.localName == TAG_REPEAT:
                params += self.parseRepeat(childElement)
            childElement = childElement.nextSibling
        return params
    
    def parseRepeat(self, xmlElement):
        params = ()
        childElement = xmlElement.firstChild
        while childElement:
            if childElement.localName == TAG_PARAM:
                params += self.parseParam(childElement)
            elif childElement.localName == TAG_PARAMREPEAT:
                params += self.parseParamRepeat(childElement)
            childElement = childElement.nextSibling
        return params
